# Copyright (C) 2000, 2001  SWsoft
# All rights reserved.
#

# Object-Oriented wrapper into HSPC::WebPage
# This class implements Abstract page class

package HSPC::AbstractPage;

use strict;


use HSPC::WebSystemLib;

use HSPC::Localization;
use HSPC::WebPage;
use Data::Dumper;
use SWsoft::protect;

use constant MIN_SECTION_SIZE => 2;

use constant _LIST_	=> 1;
use constant _VIEW_	=> 2;
use constant _EDIT_	=> 3;
use constant _ADD_	=> 4;

sub new {
	my $class = shift;
	my $d = eval "${class}::PAGE_DESCRIPTIOR" or die "ERROR: no PAGE_DESCRIPTIOR in $class class";
	return bless {
		_mode_		=> 0,
		_descriptior_	=> $d,
		_attributes_	=> { map {$_->{id}, $_}  @{$d->{attributes}} },
		_pk_		=> map {$_->{id}} grep {$_->{pk}} @{$d->{attributes}}
	}, $class;
}

sub parse_request {
	my $self = shift;

        # Script base URIs
        $self->{_script_uri_} = $ENV{'SCRIPT_NAME'};
        $self->{_script_uri_} =~ m#^(/[^/]+/)#;
        $self->{_base_uri_} = $1;

        # Open page
        $self->{_page_} = new HSPC::WebPage( { common_url => $ENV{COMPREP}->link() } );

	# Code
	$self->{_code_} = sw_param('w');
	$self->{_code_} = 'l' unless defined $self->{_code_};

	# Item ID
	$self->{item_id} = sw_param($self->{_pk_});
	if (defined $self->{item_id}) {
		$self->{item_url} = "/$self->{_pk_},$self->{item_id}";
	}
}

sub path_extensions { () };

sub path {
	my $self = shift;

	my @path = ();
	push @path, {
		title_id	=> $self->{_descriptior_}->{module_title_id},
		url		=> $self->{_base_uri_} . $self->{_descriptior_}->{dash_path}
	};
	foreach ( $self->path_extensions ) {
		push @path, $_;
	}
	push @path, {
		title_id	=> $self->{_descriptior_}->{app_title_id},
		url		=> $self->{_base_uri_} . $self->{_descriptior_}->{app_path}
	} if $self->mode == _VIEW_ || $self->mode == _EDIT_ || $self->mode == _ADD_;
	$self->{_path_}	= \@path;
}

sub tabs {
	my $self = shift;

	my $tabs = $self->{_descriptior_}->{tabs};
	my $curent = $self->current_tab;

	if (
		(
			$self->mode == _EDIT_ ||
			$self->mode == _VIEW_ ||
			$self->mode == _LIST_ && $self->show_tabs_for_list
		) &&
		$curent &&
		$tabs &&
		ref($tabs) eq 'ARRAY' &&
		$#$tabs > 0 &&
		! $self->hide_tabs
	) {
		my @arr = ();
		foreach ( @$tabs ) {
			my %rec = (
				page		=> $_->{tag},
				caption		=> $_->{caption},
				caption_id	=> $_->{caption_id}
			);
			$rec{url} = defined $_->{url} ?
				$self->custom_tab_url($_) :
				"/w,v$self->{item_url}/tab,$_->{tag}";
			push @arr, \%rec;
		}
		$self->{_tabs_} = \@arr;
		$self->{_cur_tab_} = $curent->{tag};
	}
}

sub custom_tab_url {
	my $self = shift;
	my $tab = shift;

	my $url = $tab->{url};
	$url =~ s/\?/$self->{item_id}/g;
	return $url;
}

sub hide_tabs { 0 }

sub show_tabs_for_list { 0 }

sub header {
	my $self = shift;

	$self->path;
	$self->tabs;
	
	$ENV{COMPREP}->node(edit => 1)->{title_hash}->{title} =
		$self->title;
	
	$self->{_page_}->header;
	$self->{_page_}->title;
	$self->{_page_}->tab(
		pages		=> $self->{_tabs_},
		cur_page	=> $self->{_cur_tab_}
	);
	$self->status_bar;
}

sub status_bar {
	my $self = shift;

	$self->{_page_}->{errors} = join('<br>', @{$self->errors}) if $self->errors;
	$self->{_page_}->{statuses} = join('<br>', @{$self->messages}) if $self->messages;
	$self->{_page_}->statuses();
}

sub footer {
	my $self = shift;
	$self->{_page_}->footer();
}

sub prepare_columns {
	my $self = shift;

	my @cols = ();
	my %ch = map { $_, 1 } @{$self->{_descriptior_}->{list_columns}};
	my $append_pk = 0;
	my @lookup = ();

	foreach (@{$self->{_descriptior_}->{list_columns}}) {
		my $attr = $self->{_attributes_}->{$_};
		my $col = {
			code		=> $attr->{id},
			title		=> $attr->{caption},
			title_id	=> $attr->{caption_id},
			not_escape => $attr->{not_escape}
		};
		if ( $attr->{enum} && $attr->{type} ne 'bool' ) {
			my $enum = $attr->{enum};
			$col->{type} = 'options';
			$col->{data_options} = [ map {[$_, $enum->{$_}->{caption_id} ? HSPC::Localization::string($enum->{$_}->{caption_id}) : $enum->{$_}->{caption}]} keys %$enum ];
			unless ( grep {! defined $enum->{$_}->{color} } keys %$enum ) {
				$col->{colors} = { map {$enum->{$_}->{caption_id} ? HSPC::Localization::string($enum->{$_}->{caption_id}) : $enum->{$_}->{caption}, $enum->{$_}->{color}} keys %$enum };
			}
		}
		else {
			$col->{type} = $attr->{type};
		}
		if ($attr->{drill_down}) {
			$col->{link_base} = $ENV{COMPREP}->link(alias =>
				$ENV{COMPREP}->node->{alias}, [w => 'v', $self->{_pk_} => undef]);
			$col->{link_attr} = $self->{_pk_};
			$append_pk = 1 unless exists $ch{$self->{_pk_}} || $append_pk;
		}
		$col->{default} = $attr->{default_id} ? HSPC::Localization::string($attr->{default_id}) : $attr->{default};
		$col->{hide} = 1 if $attr->{hide};
		$col->{link_base} = $attr->{link_base} if $attr->{link_base};
		push @cols, $col;

		if ($attr->{lookup}) {
			push @lookup, {
				idx	=> $#cols,
				url	=> ($attr->{lookup_alias}?
					$ENV{COMPREP}->link(alias=>$attr->{lookup_alias})
					:'').$attr->{lookup_url},
				default	=> $col->{default},
				func	=> $attr->{lookup}
			}
		}
	}
	if ( $#lookup > -1 ) {
		$self->{_lookup_} = \@lookup;
	}

	if ($append_pk) {
		push @cols, {
			code	=> $self->{_pk_},
			hide	=> 1
		};
	}

	$self->{_cols_} = \@cols;
}

sub action_bar { }

sub all_tabs {
	my $self = shift;

	my $tabs = $self->{_descriptior_}->{tabs};
	return $tabs && ref($tabs) eq 'ARRAY' && $#$tabs > -1 ? $tabs : undef;
}

sub current_tab {
	my $self = shift;

	my $tabs = $self->all_tabs or return $self->{_current_tab_} = undef;
	my $tag = sw_param('tab');
	my $tab = undef;
	foreach (@$tabs) {
		next unless $_->{tag} eq $tag;
		$tab = $_;
	}

	return $self->{_current_tab_} = $tab ? $tab : $tabs->[$self->default_tab_idx];
}

sub default_tab_idx {
	my $self = shift;

	my $tabs = $self->all_tabs or return 0;
	for my $i (0 .. $#$tabs) {
		return $i if $tabs->[$i]->{default};
	}
	return 0;
}

sub sections_on_tab {
	my $self = shift;

	my $tab = $self->{_current_tab_} ? $self->{_current_tab_} : $self->current_tab;

	if ( defined $tab ) {
		unless ( defined $tab->{sections} ) {
			# empty set - no sections
			return ();
		}
		elsif ( ref($tab->{sections}) eq 'ARRAY' ) {
			# sections in ARRAYREF
			return @{$tab->{sections}};
		}
		else {
			# sections in SCALAR
			return $tab->{sections};
		}
	}
	else {
		# No tabs ever defined - return all tabs
		return (0 .. $#{$self->{_descriptior_}->{sections}});
	}
}

sub visible_attributes {
	my $self = shift;

	return @{$self->{_visible_attributes_}} if defined $self->{_visible_attributes_};

	my %valid = map { $_, 1 } $self->sections_on_tab;


	my @arr =
		grep { defined $self->{$_->{id}} }		# Attribute data must be loaded my MT
		grep { $valid{$_->{section}} }			# Section must be on current tab
		grep { $_->{id} ne 'title' }			# Exclude title
		grep { ! $_->{hide} }				# Exclude hidden attributes
		@{$self->{_descriptior_}->{attributes}};

	$self->{_visible_attributes_} = \@arr;

	return @arr;
}

sub hide_sections {
	my $self = shift;

	return $self->{_hide_sections_} if defined $self->{_hide_sections_};

	# Visible sections
	my @sections = map { $_->{section} } $self->visible_attributes;

	# Calculate size of the smallest section
	# Also check for intersecting sections
	my $recent = $sections[0];	# recently section processed seciton
	my $ordered = 1;		# flag - 1 if are well-ordered
	my %seen = ();
	foreach ( @sections ) {
		$seen{$_} = 0 unless exists $seen{$_};
		$seen{$_}++;
		$ordered = 1 unless $_ >= $recent;
		$recent = $_;
	}
	my @us = keys %seen;
	my @tmp = sort values %seen;
	my $min = shift @tmp;

	return $self->{_hide_sections_} =
		$ordered &&			# Hide section headers if they are not well ordered
		$#us >= 1 &&			# Hide sections unless there is more than 1 section
		$min >= MIN_SECTION_SIZE ?	# Hide sections if they are too small
		0 : 1;
}

sub show_section_header {
	my ($self, $section) = @_;

	$self->hide_sections or do {
		my $title_id = $self->{_descriptior_}->{sections}->[$section];
		$self->{_page_}->view_info_folder (
			title_id	=> $title_id
		) if $title_id;
	};
	return $section;
}

sub show_text_box {
	my ($self, $atr) = @_;

	my $value = $self->{$atr->{id}};

	if ($atr->{enum}) {
		$value =
			$atr->{enum}->{$value}->{caption_id} ?
			HSPC::Localization::string($atr->{enum}->{$value}->{caption_id}) :
			$atr->{enum}->{$value}->{caption};
	}

	if ( $self->mode == _VIEW_ ) {
		my %prop = (
			value	=> $value
		);
		unless ( $atr->{wide} ) {
			$prop{title} = $atr->{caption_item}||$atr->{caption};
			$prop{title_id} = $atr->{caption_item_id}||$atr->{caption_id};
		}
		else {
			$prop{is_big} = 1;
		}
		$self->{_page_}->view_info_text ( %prop );
	}

}

sub show_item {
	my $self = shift;
	
	$ENV{COMPREP}->node(choose => 1, alias => $ENV{COMPREP}->node->{alias} . '_item');
	$self->mode(_VIEW_);

	$self->item_data;

	my @attr = $self->visible_attributes;

	$self->header;

	$self->{_page_}->view_info_header;
	my $recent = $self->show_section_header($attr[0]->{section});

	foreach ( @attr ) {
		$recent = $self->show_section_header($_->{section}) unless $recent == $_->{section};
		$self->show_text_box($_);
	}
	$self->{_page_}->edit_view_hidden(
		view_name => $self->{_pk_},
		value => $self->{item_id},
	);
	$self->{_page_}->view_info_footer;
	$self->{_page_}->view_info_button (
		show_cancel	=> 1,
		right_buttons => $self->{_descriptior_}->{view_right_buttons},
#		show_edit	=> $self->{allow_edit},
#		url_ext		=> $url_ext
	);

}

sub show_edit {
	my $self = shift;

	$self->mode(_EDIT_);
	$self->header;
	#### TODO
}

sub show_add {
	my $self = shift;
	
	$ENV{COMPREP}->node(choose => 1, alias => $ENV{COMPREP}->node->{alias} . '_new');

	$self->mode(_ADD_);
	$self->header;
	#### TODO
}

sub title {
	my $self = shift;

	if ( $self->mode == _ADD_ ) {
		return HSPC::Localization::string($self->{_descriptior_}->{add_title_id});
	}
	elsif ( $self->mode == _LIST_ ) {
		return HSPC::Localization::string($self->{_descriptior_}->{app_title_id});
	}
	elsif ( $self->mode == _VIEW_ ) {
		return $self->item_data->{_item_name_};
	}
}

sub devscreen_id {
	my $self = shift;

	if ( $self->mode == _ADD_ ) {
		return $self->{_descriptior_}->{add_devscreen_id};
	}
	elsif ( $self->mode == _LIST_ ) {
		return $self->{_descriptior_}->{app_devscreen_id};
	}
	elsif ( $self->mode == _VIEW_ ) {
		return $self->item_data->{_item_devscreen_id_};
	}
}

sub show_list {
	my $self = shift;

	$self->mode(_LIST_);

	$self->header;

	$self->{_page_}->{res_itemstitle_id} = $self->{_descriptior_}->{items_title_id};
	$self->{_page_}->{res_itemtitle_id} = $self->{_descriptior_}->{item_title_id};

	$self->prepare_columns unless $self->{_cols_};

	$self->{_browse_} = $self->{_page_}->prepare_browse( cols => $self->{_cols_} );
	$self->list_data;
	$self->action_bar;
	$self->{_page_}->browse (
		browse_info	=> $self->{_browse_},
		@_
	);
}

sub handle_custom_args {
	return undef;
}

sub handler {
	my $self = shift;

	$self->parse_request;

	if ( $self->{_code_} eq 'v' ) {
		$self->show_item;
	}
	elsif ( $self->{_code_} eq 'e' ) {
		if (defined $self->{item_id}) {
			$self->show_edit;
		}
		else {
			$self->show_add;
		}
	}
	elsif ($self->{_code_} eq 's') {
		$self->{_page_}->redirect() if sw_argparam('item_cancel');
		$self->restore_args;
		if ( $self->check ) {
			if (defined $self->{item_id}) {
				if ($self->update) {
					$self->{_page_}->redirect($self->{_descriptior_}->{updated_id});
					return;
				}
			}
			else {
				if ($self->add) {
					$self->{_page_}->redirect($self->{_descriptior_}->{added_id});
					return;
				}
			}
		}
		if (defined $self->{item_id}) {
			$self->show_edit;
		}
		else {
			$self->show_add;
		}
	}
	elsif ($self->handle_custom_args()){
		## all actions will be in handle_custom_args
	}
	else {
		$self->show_list;
	}
	$self->footer if $self->mode;
}

sub restore_args { }

sub check {
	return 1;
}

sub throw {
	my $self = shift;
	$self->{_errors_} = [] unless $self->{_errors_};
	push @{$self->{_errors_}}, @_;
}

sub inform {
	my $self = shift;
	$self->{_messages_} = [] unless $self->{_messages_};
	push @{$self->{_messages_}}, @_;
}

sub errors {
	my $self = shift;
	return $self->{_errors_};
}

sub messages {
	my $self = shift;
	return $self->{_messages_};
}

sub update {
	return 1;
}

sub add {
	return 1;
}

sub showpage {
	my $class = shift;
	my $self = new($class);

	$self->handler;

	return {STATUS => 'OK', CONTENT => $self->{_page_}->{edit_out}};
}

sub mode {
	my $self = shift;
	my $arg = shift;

	if (defined $arg) {
		$self->{_mode_} = $arg;
	}
	return $self->{_mode_};
}

sub item_data {
	my $self = shift;
	if ( $self->{_item_fetched_} ) {
		return $self;
	}
	else {
		$self->{_item_fetched_} = 1;
		return undef;
	}
}

sub list_data {
	my $self = shift;

	if ( $self->{_lookup_} ) {
		my $rows = $self->{_browse_}->{data};
		foreach my $lookup ( @{$self->{_lookup_}} ) {
			foreach my $row ( @$rows ) {
				my $id = $row->[$lookup->{idx}];
				if (defined $id) {
					my $val = &{$lookup->{func}}($self, $id);
					$val = '' unless defined $val;
					my $url = $lookup->{url};
					$url =~ s/\?/$id/g;
					$row->[$lookup->{idx}] = sprintf '<a href="%s">%s</a>', $url, $val;
				}
				else {
					$row->[$lookup->{idx}] = $lookup->{default};
				}
			}
		}
	}
}

sub summary_data {
	return { };
}

sub summary {
	my $self = shift;
	return $self->{_summary_data_} if $self->{_summary_data_};
	return $self->{_summary_data_} = $self->summary_data;
}

sub to_string {
	my $self = shift;
	return Dumper($self);
}

1;

__END__
