# baza@sw.ru

package HSPC::DataRequest;
use strict;
use SWsoft::protect;	# Mark this module for encryption upon build
use POSIX qw(locale_h);

use HSPC::Logger qw(sw_die);
use HSPC::Localization qw(:DEFAULT string string_lang safe_setlocale reset_locale);
use HSPC::Localization::Date qw(time_to_secs);

# => cols look to WebPage->prepare_browse() API
sub new{
	my $self = {class=>shift,@_};
	bless ($self,$self->{class});
	
	if(ref($self->{cols}->[0]) ne 'HASH'){
		my $cols;
		foreach my $col(@{ $self->{cols} }){
			push @$cols,{code=>$col};
		}
		$self->{cols} = $cols;
	}
	
	# set indexes #
	my $index=0;
	map{$ _->{index}=$index++ } @{$self->{cols}};
			
	return $self;
}

# > self
# > col_code
# < col
sub get_col_by_code{
	my $self = shift;
	my $col_code = shift;
			
	foreach my $col (@{$self->{cols}}){						
		return $col if($col->{code} eq $col_code);
	}	
	return undef;
}

# > self
# > col_code
# < index
sub get_index_by_code{
	my $self = shift;
	my $col_code = shift;
	
	foreach my $col (@{$self->{cols}}){
		return $col->{index} if( $col->{code} eq $col_code);
	}	
	return -1;
}

# > self
sub _add_col{
	my $self = shift;
	my %h=(
		index => $#{ $self->{cols} }+1,
	@_);			
	push @{$self->{cols}},\%h;
				
	return \%h;
}

# > self
# >= col_args
sub add_col{
	my $self = shift;
	my %h=(@_);	
	sw_die('code=> undefined') unless $h{code};
				
	my $col = $self->col( $h{code} ) || $self->_add_col(%h);
	return $col->{index};
}

# > self
# > col_code
# < col
sub col{
	my $self = shift;
	my $col_code = shift;
	
	foreach my $col (@{$self->{cols}}){
		return $col if( $col->{code} eq $col_code);
	}	
	return undef;
}

# > self
# > col_code
# < col
sub value{
	my $self = shift;
	my $col_code = shift;
	
	foreach my $col (@{$self->{cols}}){
		return $col->{value} if( $col->{code} eq $col_code);
	}	
	return undef;
}

## Getter/Setter for complex filter values
## 1st parameter - filter column code
## 2nd parameter - values to set
## < return ref to hash with filter values
sub filter_value {
	my $self = shift;
	my $col_code = shift;
	my $value = shift;

	my %result;	
	foreach my $col (@{$self->{cols}}){
		next if( $col->{code} ne $col_code);

		if (
			$col->{for_frequency} && (
				$col->{type} eq 'datetime' || $col->{type} eq 'date'
			)
		) {
			if (
				$value && ref $value eq 'HASH' &&
				exists $value->{value_from} &&
				exists $value->{value_to}
			) {
				$col->{value_from} = $value->{value_from};
				$col->{value_to} = $value->{value_to};
			}
			$result{value_from} = $col->{value_from};
			$result{value_to} = $col->{value_to};
		} elsif ( $col->{for_filter} ) {
			if ( defined $value ) {
				$col->{value} = $value eq '' ? undef : $value;
			}
			return $col->{value};
		} elsif (
			$col->{for_range} && (
				$col->{type} eq 'datetime' || $col->{type} eq 'date'
			)
		) {
			if (
				$value && ref $value eq 'HASH' &&
				exists $value->{value_from} &&
				exists $value->{value_to}
			) {
				$col->{value_from} = $value->{value_from};
				$col->{value_to} = $value->{value_to};
			}
			$result{value_from} = $col->{value_from};
			$result{value_to} = $col->{value_to};
		}
	}	
	return \%result;
}

# < return ref to $self->{data} attribute
sub data{
	my $self = shift;
	$self->{data} = shift if (@_);
	return $self->{data};
}

# < return data size
sub total_items{
	my $self = shift;
	return $self->{total_items};
}

# > self
# >= code
# >= PROPS
sub set_col{
	my $self = shift;
	my %h=(@_);
	
	my $col = $self->col($h{code});	
	if($col){	
		foreach(keys %h){	
			$col->{$_} = $h{$_};
		}
	}else{
		$col = $self->_add_col(%h);		
	}		
	
	return $col;
}

# > self
# > col_code
# > col_value
sub set_where{
	my ($self,$code,$value) = @_;
		
	#my $col = $self->get_col_by_code($code);	
	my $col = $self->col($code)||$self->_add_col(code=>$code);

	$col->{value} = $value;	
	$col->{for_where} = 1;	
	return $col;	
}

# > self
# > col_code
# > col_value
sub unset_where{
	my ($self,$code) = @_;
		
	#my $col = $self->get_col_by_code($code);	
	my $col = $self->col($code);

	if ($col) {
		delete $col->{value};
		$col->{for_where} = 0;
	}
	return $col;	
}

#>= attrs : string with attributes names, example :  "count,sum,total"
#>= title_attr : string with attribute name, uses for "Total" text
sub insert_total{
	my $self = shift;
	my %h=(@_);
	
	return unless $self->{total_items};	
	my %total_hash;

	my @attrs = split(',',$h{attrs});	

	my %attrs;
	my $vendor_col = '';
	foreach my $attr_name(@attrs){			
		$attrs{$attr_name} = 1;
		
		my $attr = $self->col($attr_name);		
		die qq[ERROR: no attr with code "$attr_name"] unless $attr;

		if ($attr->{type} eq 'money' && $attr->{vendor_col} ) {
			$vendor_col = $attr->{vendor_col};
		}

		my $s=0;
		map { $s+=$_->[$attr->{index}] } @{ $self->{data}  };
		$total_hash{$attr_name} = $s;
	}
	
	my @total_array;
	foreach my $attr(@ {$self->{cols}} ){
		push @total_array,defined( $total_hash{$attr->{code}} )?$total_hash{$attr->{code}}:undef;
	}

	if ($vendor_col) {
		my $vendor_idx = $self->get_index_by_code($vendor_col);
		## Currency sign will be eqaul to the sign from the first row
		## Totals should never be used in mixed lists with mixed currencies
		$total_array[ $vendor_idx ]  = $self->{data}->[0]->[$vendor_idx];
	}

	if(defined( $h{title_attr} )){
		my $attr = $self->col($h{title_attr});
		die qq[ERROR: no attr with code "$h{title_attr}"] unless $attr;
		$total_array[ $attr->{index} ]  = string('dr_total');
	}
	
	push @{$self->{data}},\@total_array;	
	
	$self->{totalrec_cols} = \%attrs;
}

#### autosort data 
## >> self
sub sort{
	my $self = shift;
	
	my @data = $self->sort_data( $self->{sort_col_code},$self->{sort_revers},$self->data );
	$self->{data} = \@data;
}

#-----------------------------------------------------------------------------

=pod

B<sort_data (...)> - Sorts data array before using B<browse> method 
if used array, not database access. If database used this method not needed.
Returns sorted array (array structure is like to input data $browse_data).

Parameters:
    
    $sort_col_code - code of column, used to sort.
    $reverse - set to "1" if need to sort in reversed order.
    $browse_data - data to sort. See B<prepare_browse & browse> for details.

=cut

sub sort_data
{
    my $self = shift;
    my $sort_col_code = shift;
    my $reverse = shift;
    my $browse_data = shift;
    
	## prepare locale for correct sorting
	use locale;
	my $lang_info = HSPC::Localization::get_language(HSPC::Localization::current_lang());
	my $locale = safe_setlocale(LC_COLLATE, $lang_info->{posixlocale});

    #
    # Get sort column number by it's code...
    #
    my $sort_column_num = $self->get_index_by_code($sort_col_code);
    $sort_column_num = 0 if $sort_column_num == -1;
    
    #
    # ... and sort by this column
    #
    my @new_data;
    if (    $self->{cols}->[$sort_column_num]->{type} eq 'char'
         || $self->{cols}->[$sort_column_num]->{type} eq 'q'
         || $self->{cols}->[$sort_column_num]->{type} eq 'email'
         || $self->{cols}->[$sort_column_num]->{type} eq 'country'
         || $self->{cols}->[$sort_column_num]->{type} eq 'state'
         || $self->{cols}->[$sort_column_num]->{type} eq 'wide'
         || $self->{cols}->[$sort_column_num]->{type} eq 'options'
       ) 
    {
        if ($reverse) {
                @new_data = sort { uc($$b[$sort_column_num]) cmp uc($$a[$sort_column_num]) } @{$browse_data};
        } else {
                @new_data = sort { uc($$a[$sort_column_num]) cmp uc($$b[$sort_column_num]) } @{$browse_data};
        }
    }
	elsif ( $self->{cols}->[$sort_column_num]->{type} eq 'datetime' ) {
        if ($reverse) {
				@new_data = sort { time_to_secs($$b[$sort_column_num]) <=> time_to_secs($$a[$sort_column_num]) } @{$browse_data};
        } else {
				@new_data = sort { time_to_secs($$a[$sort_column_num]) <=> time_to_secs($$b[$sort_column_num]) } @{$browse_data};
        }
	}
    else 
    {
        if ($reverse) {
                @new_data = sort { $$b[$sort_column_num] <=> $$a[$sort_column_num] } grep {$_->[$sort_column_num] == $_->[$sort_column_num]} @{$browse_data};
        } else {
                @new_data = sort { $$a[$sort_column_num] <=> $$b[$sort_column_num] } grep {$_->[$sort_column_num] == $_->[$sort_column_num]} @{$browse_data};
        }
    }
    
	## restore previous locale
	reset_locale(LC_COLLATE, $locale);

    # Return...
    return @new_data;
}

## Used for manual filters, i.e. when data gathered not from SQL,
## but any other way, so auto-filters not applied.
## >> self
sub apply_filters
{
    my $self = shift;
	my @filtered_data;
	my $rows_count;

	my $filter;
	my $row;
	my $skip;
	foreach $row (@{$self->{data}}) {
		$skip = 0;
		## Check filter for each line
		foreach $filter (@{$self->{filters}}) {
			next if !$filter->{value}; ## Skip empty filter
			next if $filter->{for_search} != 1; ## Support only Search
			if (   $filter->{type} eq 'char'
				|| $filter->{type} eq 'q'
				|| $filter->{type} eq 'email'
				|| $filter->{type} eq 'country'
				|| $filter->{type} eq 'state'
				|| $filter->{type} eq 'wide'
				|| $filter->{type} eq 'options')
			{
				## Apply filter
				if (index (lc($row->[$filter->{index}]), lc($filter->{value})) < 0)
				{
					$skip = 1;
					last;
				}
			}
			elsif ($filter->{type} eq 'money') {
				## Apply filter
				if ($row->[$filter->{index}] != $filter->{value}) {
					$skip = 1;
					last;
				}
			}
			else {
				## Skip not supported filter types
				next;
			}
		}

		if (!$skip) {
			$rows_count++;
			push @filtered_data, $row;
		}
	}

	$self->{data} = \@filtered_data;
	$self->{count} = $self->{total_items} = $rows_count;
}

=pod

B<mem_array()> - Prepares data in memory array for printing with B<browse>.
Sorts memory array, limits array according to current page numer and page
size. Normally B<mem_array> is used prior to making B<browse> method call
when custom memory array is used in contrast to database access.

Returns:        nothing.
Parameters:     none

=cut

sub mem_array {
	my $self = shift;

	# Sorting
	my @rows = $self->sort_data(	
		$self->{sort_col_code},
		$self->{sort_revers},
		$self->{data}
	);
	
	# Total items
	$self->{total_items} = $#rows + 1;
	
	# Limit rows according to current page and page size
	my @frame = ();
	for my $i ( $self->{from} - 1 .. $self->{from} + $self->{count} - 2 ) {
		last if $i > $#rows;
		push @frame, $rows[$i];
	}

	# Save results
	$self->{data} = \@frame;
}

=pod

B<sort_hash_data (...)> - Sorts data hash before using B<browse> method 
if not used database access. If database used this method not needed.
Returns sorted array (array structure is like to input data $browse_data).

Parameters:
    
    $browse_info - See B<prepare_browse & browse> for details.
    $browse_data - data to sort. See B<prepare_browse & browse> for details.

=cut
sub sort_hash_data
{
    my $self = shift;
    my $browse_info = shift;
    my $browse_data = shift;
    
    return unless $browse_data;
    
    my $sort_col_code = $browse_info->{sort_col_code};
    my $reverse = $browse_info->{sort_revers};
    
    #
    # Get sort column number by it's code
    #
    my $sort_column_num = $self->get_index_by_code($sort_col_code);
    $sort_column_num = 0 if $sort_column_num == -1;
    
    #
    # Sort by this column
    #
    my @new_data;
    if (    $self->{cols}->[$sort_column_num]->{type} eq 'char'
         || $self->{cols}->[$sort_column_num]->{type} eq 'q'
         || $self->{cols}->[$sort_column_num]->{type} eq 'email'
         || $self->{cols}->[$sort_column_num]->{type} eq 'country'
         || $self->{cols}->[$sort_column_num]->{type} eq 'state'
         || $self->{cols}->[$sort_column_num]->{type} eq 'wide'
       ) 
    {
        if ($reverse) {
                @new_data = sort { uc($b->{"$sort_col_code"}) cmp uc($a->{"$sort_col_code"}) } @{$browse_data};
        } else {
                @new_data = sort { uc($a->{"$sort_col_code"}) cmp uc($b->{"$sort_col_code"}) } @{$browse_data};
        }
    } else {
        if ($reverse) {
                @new_data = sort { $b->{"$sort_col_code"} <=> $a->{"$sort_col_code"} } @{$browse_data};
        } else {
                @new_data = sort { $a->{"$sort_col_code"} <=> $b->{"$sort_col_code"} } @{$browse_data};
        }
    }
    
    # Get array from 'from' to 'from + count'

    # Return...
    return @new_data;
}

sub cut {
	my $self = shift;
	
	return undef unless $self->data;

	my $from = $self->{from} || 1;
	my $count = $self->{count} || 0;
	$from--;

	splice (@{$self->data}, 0, $from) if $from;
	splice (@{$self->data}, $count) if $count;
	return undef;
}

sub recalculate_total_items {
	my $self = shift;
	$self->{total_items} = $self->data ? scalar (@{$self->data}) : 0;
}

1;
__END__

Object Attributes

	data : ref to array
	total_items :
	totalrec_added : 
