package HSPC::MT::Permissions;

use strict;

use HSPC::WebDB;
use HSPC::Localization;
use HSPC::MT::Role;

use SWsoft::protect;	# Mark this module for encryption upon build

use vars qw(
	@ISA @EXPORT
);
                                         
@ISA = qw(Exporter);
@EXPORT = qw(
	&defaults
	&load
	&update
	&update_defaults
);

sub load {
	my $roles_hashref = HSPC::MT::Role::get_roles(EXCLUDE_ADMINS => 1, @_);
	my $ret = HSPC::WebDB::select_hash_all( qq {
			SELECT sc_id, BIT_OR(access_mask)
			FROM access_control_list
			WHERE
				role_id = ?
				AND (account_no = ? OR account_no = 0)
			GROUP BY sc_id
		},
		[keys %$roles_hashref],
		$ENV{security_obj}->account_no
	);
	return $ret;
}

sub update {
	my $h = shift;
	
	my $roles_hashref = HSPC::MT::Role::get_roles(EXCLUDE_ADMINS => 1);
	my $account_no = $ENV{security_obj}->account_no;

	foreach my $role_id (keys %$roles_hashref) {
		# First revoke all ('%') permissions granted to this role_id
		_revoke ($account_no, $role_id, '%');
		# Next grant new permissions
		my $list = $h->{$role_id} || next;
		foreach my $sc_id (keys %$list) {
			_grant ($account_no, $role_id, $sc_id, $list->{$sc_id});
		}
	}
}

sub update_defaults {
	my $h = shift;
	my %args = (
		ACCOUNT_TYPE	=> undef,
		@_
	);
	
	# Argument required
	defined $args{ACCOUNT_TYPE} or die "Argument ACCOUNT_TYPE is required";
	
	# Do not allow anyone except HSP change defaults
	$ENV{security_obj}->account_type == 0 or do {
		return;
	};
	
	my $roles_hashref = HSPC::MT::Role::get_roles(EXCLUDE_ADMINS => 1, ACCOUNT_TYPE => $args{ACCOUNT_TYPE});

	foreach my $role_id (keys %$roles_hashref) {
		# First revoke all ('%') permissions granted to this role_id
		_revoke (0, $role_id, '%');
		# Next grant new permissions
		my $list = $h->{$role_id} || next;
		foreach my $sc_id (keys %$list) {
			_grant (0, $role_id, $sc_id, $list->{$sc_id});
		}
	}
}

sub update_role {
	my $list = shift;
	my %args = (
		ACCOUNT_TYPE	=> undef,
		ROLE_ID	=> undef,
		@_
	);
	
	# Arguments required
	defined $args{ACCOUNT_TYPE} or die "Argument ACCOUNT_TYPE is required";
	defined $args{ROLE_ID} or die "Argument ROLE_ID is required";
	
	# Do not allow anyone except HSP change permissions
	return 
		if $ENV{security_obj}->account_type != 0;
			
	# First revoke all ('%') permissions granted to this role_id
	_revoke (0, $args{ROLE_ID}, '%');

	# Next grant new permissions
	foreach my $sc_id (keys %$list) {
		_grant (0, $args{ROLE_ID}, $sc_id, $list->{$sc_id});
	}
	
}

sub _revoke {
	select_run( q {
		DELETE FROM access_control_list
		WHERE
			account_no = ?
			AND role_id = ?
			AND sc_id LIKE ?
		}, @_
	);
}

sub _grant {
	select_run( q {
			REPLACE access_control_list (account_no, role_id, sc_id, access_mask)
			VALUES (?, ?, ?, ?)
		},
		@_
	);
}

1;

__END__
