package HSPC::MT::Person;

use strict;

use HSPC::WebDB;
use HSPC::MT::Entity;
use HSPC::MT::Address;
use HSPC::MT::Person::Role;
use SWsoft::protect;	# Mark this module for encryption upon build

my $debug = 0;

sub get_person {
	my $person_id = shift;
	my $person = HSPC::WebDB::select_hash(q{select * from person where person_id=?},$person_id);
	return $person;
}

sub get_person_fullname {
	my $self = shift;
	my $person_id = shift;
	return HSPC::WebDB::select_result(q{SELECT
					rtrim( concat(
						if(strcmp(p.prefix, ''), concat(p.prefix, ' '), ''),
						if(strcmp(p.fname,  ''), concat(p.fname,  ' '), ''),
						if(strcmp(p.mname,  ''), concat(p.mname,  ' '), ''),
						if(strcmp(p.lname,  ''), concat(p.lname,  ' '), ''),
						if(strcmp(p.suffix, ''), concat(p.suffix, ' '), '')
					) )
				from person p
				where p.person_id=?},$person_id);
}

# return array hash for account
# hash keys: person_id, full_name, passport_email
sub get_arr_account_staff {
	sw_die("Something REALLY bad happened. Ask baza@ or dkoles@ for info.");
}


sub get_status {
	my ($person_id, $account_no) = @_;
	my $person = HSPC::WebDB::select_hash(q{select status from person_account where person_id = ? and account_no = ?}, $person_id, $account_no);
	return $person;
}

sub get_person_by_mail {
	my $class = shift;
	my $person_email = shift;
	my $person = HSPC::WebDB::select_hash(q{select * from person where email=?},$person_email);
	return $person;
}

sub check_count {
	my $class = shift;
	my $person_email = shift;
	return HSPC::WebDB::select_result(q{SELECT count(person_id) from person where email=?},$person_email);
}

sub load_person {
	my $self = shift;

	my ($address, $person);
	$person = get_person($self->{person_id}) if (defined $self->{person_id});
	$address = HSPC::MT::Address::get_address($person->{address_id}) if (defined $person->{address_id});

	#### Init obj ####
	#map { $self->{$_} = $person->{$_} } keys %$person;
	#map { $self->{$_} = $address->{$_} } keys %$address;

	#### Init obj ####
	$self->{person_id}=$person->{person_id};
	$self->{name_title}=$person->{prefix};
	$self->{name_first}=$person->{fname};
	$self->{name_middle}=$person->{mname};
	$self->{name_last}=$person->{lname};
	$self->{name_suffix}=$person->{suffix};
	$self->{email}=$person->{email};
	$self->{comment}=$person->{comment};

	$self->{address1}=$address->{address1};
	$self->{address2}=$address->{address2};
	$self->{city}=$address->{city};
	$self->{state}=$address->{state};
	$self->{state_alt}=$address->{state_alt};
	$self->{country}=$address->{country};
	$self->{zip}=$address->{zip};
	$self->{phone}=$address->{phone};
	$self->{phone_src}=$address->{phone_src};

	if ($self->{account_no}) {
		my $pa = get_status($self->{person_id}, $self->{account_no});
		$self->{person_status} = $pa->{status};
	}
}

sub insert_person {
	HSPC::WebDB::select_run(q{INSERT INTO person (email,prefix,fname,mname,lname,suffix,address_id,comment,created_by_acct_no) VALUES (?,?,?,?,?,?,LAST_INSERT_ID(),?,?)},@_, $ENV{security_obj}->account_no);
	return HSPC::WebDB::select_result(q{SELECT LAST_INSERT_ID()});
}

sub insert {
	sw_die("Something REALLY bad happened.\nAsk baza@ or dkoles@ for more info.");
}

sub update_person { HSPC::WebDB::select_run(q{UPDATE person set prefix=?, fname=?, mname=?, lname=?, suffix=?, email=?, comment=? where person_id=?},@_); }

sub update_status {
	HSPC::WebDB::select_run( q{
			REPLACE person_account(account_no, person_id, status)
			values (?, ?, ?)
		},
		@_
	);
}

sub update {
	my $self = shift;
	my $account_no = shift;
	my %args = (
		UPDATE_ROLES	=> 1,
		@_,
	);

	warn "DEBUG: updating person..." if $debug;

	
	my $err = $ENV{security_obj}->change_person_pwd(
		email=>$self->{email} , pwd=>$self->{password}
	);
	return $err if $err;

	my $address_id = HSPC::MT::Address::get_id_from_person($self->{person_id});
	HSPC::MT::Address::update($self->{address1},$self->{address2},$self->{city},$self->{state},$self->{state_alt},$self->{zip},$self->{country},$self->{phone},$self->{phone_src},$address_id) if ($address_id);

	update_person($self->{name_title},$self->{name_first},$self->{name_middle},$self->{name_last},
	$self->{name_suffix},$self->{email},$self->{comment},$self->{person_id});

	if ( $args{UPDATE_ROLES} ) {
		HSPC::MT::Person::Role::delete_person_role($account_no,$self->{person_id});
		HSPC::MT::Person::Role::insert_person_roles($account_no,$self->{person_id},$self->{role});
	}

	return $self->{person_id};
}

sub delete {
	my $self = shift;
	my $account_no = shift;
	#
	HSPC::WebDB::select_run(q{update person_account set status = 255 where account_no = ? and person_id = ?},$account_no,$self->{person_id});
}

# > class
# > browse_info : {}
# < list
sub get_company_staff {
	my $class = shift;
	my $info = shift;
	if (defined $info->{client_account_id}){
		my $roles = HSPC::MT::Entity::get_account_roles($info->{client_account_id});
		my $join_sql = '';
		my $fields_sql = '';
		foreach my $role(@$roles){
			$join_sql .= ' left join person_account_role par'.$role->[0].' on p.person_id = par'.$role->[0].'.person_id  and par'.$role->[0].'.account_no = pa.account_no and par'.$role->[0].'.role_id='.$role->[0].' left join role r'.$role->[0].' on par'.$role->[0].'.role_id=r'.$role->[0].'.role_id ';
			$fields_sql .= 'ifnull(r'.$role->[0].'.name,""),"#",';
		}
		$fields_sql = ' concat('.$fields_sql.'"")' if ($#{$roles}>=0);
		####
		HSPC::WebDB::select_data(
		tables=>'person p, person_account pa'.$join_sql,
		gate_cols=>{idp=>'p.person_id',name=>'concat(p.prefix," ",p.fname," ",p.mname," ",p.lname," ",p.suffix)', role=>$fields_sql, email=>'p.email'},
		where=>'pa.status!=255 and pa.account_no='.$info->{client_account_id}.' and pa.person_id=p.person_id ',
		request=>$info);
	}
}

# Returns list of persons
sub get_person_list {
	my ($class, $info) = @_;

	# retreive security object
	my $so = $ENV{security_obj};

	# For HSP and Resellers only
	return unless $so && $so->{account_type} != 3;

	# Limit Reseller's visibility scope
	my $scope_constraint = '';
	my $where = '';
	$so->account_type == 0 or do {
		my $this_account_no = HSPC::WebDB::quote($so->{account_no});
		$scope_constraint = qq {
			LEFT OUTER JOIN person_account x
				ON x.person_id = p.person_id
				AND x.status != 255
			LEFT OUTER JOIN account ac
				ON ac.account_no = x.account_no
		};
		$where = qq {
			ac.vendor = $this_account_no OR ac.account_no = $this_account_no OR p.created_by_acct_no = $this_account_no
		};
	};

	my $tables = qq {
			person p
			$scope_constraint
			LEFT OUTER JOIN address a ON a.address_id = p.address_id
	};

HSPC::WebDB::select_data (
		request		=> $info,
		tables		=> $tables,
		where		=> $where,
		distinct	=> $scope_constraint ? 1 : 0,
		gate_cols	=> {
			p			=> 'p.person_id',
			name		=> q {
							rtrim( concat(
								if(strcmp(p.fname,  ''), concat(p.fname,  ' '), ''),
								if(strcmp(p.mname,  ''), concat(p.mname,  ' '), ''),
								if(strcmp(p.lname,  ''), concat(p.lname,  ' '), ''),
								if(strcmp(p.suffix, ''), concat(p.suffix, ' '), '')
							) )
						   },
			email		=> "ifnull(p.email,'')",
			phone		=> 'a.phone',
			state		=> "ifnull(a.state,'')",
			state_alt		=> "ifnull(a.state_alt,'')",
			country		=> 'a.country',
		},
		debug		=> $debug
	);
}

# Returns list of staff members for this compamy
sub get_staff_list {
	my ($class, $info) = @_;

	# retreive security object
	my $so = $ENV{security_obj};
	my $this_account_no = HSPC::WebDB::quote($so->{account_no});

	my $tables = qq {
			person p
			INNER JOIN person_account x
				ON x.person_id = p.person_id
				AND x.account_no = $this_account_no
				AND x.status != 255
			INNER JOIN person_account_role r
				ON r.person_id = x.person_id
				AND r.account_no = x.account_no
			LEFT OUTER JOIN address a ON a.address_id = p.address_id
	};

	HSPC::WebDB::select_data (
		request		=> $info,
		tables		=> $tables,
		distinct	=> 1,
		gate_cols	=> {
			p			=> 'p.person_id',
			status		=> 'x.status',
			name		=> q {
							rtrim( concat(
								if(strcmp(p.prefix, ''), concat(p.prefix, ' '), ''),
								if(strcmp(p.fname,  ''), concat(p.fname,  ' '), ''),
								if(strcmp(p.mname,  ''), concat(p.mname,  ' '), ''),
								if(strcmp(p.lname,  ''), concat(p.lname,  ' '), ''),
								if(strcmp(p.suffix, ''), concat(p.suffix, ' '), '')
							) )
						   },
			email		=> "ifnull(p.email,'')",
			phone		=> 'a.phone',
			role		=> 'r.role_id',
			last_login	=> 'p.last_login',
		},
	);
}

# Returns list of persons accounts
# Arguments:
#   person_id
sub get_person_accounts {
	my ($class, $info) = @_;

	my $person_id = HSPC::WebDB::quote($info->{person_id});

	HSPC::WebDB::select_data (
		request		=> $info,
		tables		=>  q {
			account a
			INNER JOIN person_account x ON x.account_no = a.account_no
		},
		gate_cols	=> {
			a			=> 'a.account_no',
			name		=> 'a.name',
			type		=> 'a.account_type',
			vendor		=> 'a.vendor',
			is_coprorate => 'a.is_coprorate',
			status		=> 'x.status',
		},
		where		=> qq {
			x.person_id = $person_id
			AND x.status != 255
		},
	);
}


# Returns list of persons customers
# Arguments:
#   person_id
sub get_person_customers {
	my ($class, $info) = @_;

	my $person_id = HSPC::WebDB::quote($info->{person_id});

	HSPC::WebDB::select_data (
		request		=> $info,
		tables		=>  q {
			account a
			INNER JOIN person_account x ON x.account_no = a.account_no
		},
		gate_cols	=> {
			a			=> 'a.account_no',
			name		=> 'a.name',
			type		=> 'a.account_type',
			vendor		=> 'a.vendor',
			is_coprorate => 'a.is_coprorate',
			astatus		=> 'a.status',
			status		=> 'x.status',
		},
		where		=> qq {
			x.person_id = $person_id
			AND x.status != 255
			AND a.status != 255
		},
	);
}

# Counts number of resellers filtered by specified criteria
sub count {
	my $class = shift;
	my $what = shift || 'ALL';

	# retreive security object
	my $so = $ENV{security_obj};

	# For HSP and Resellers only
	return undef unless $so && $so->{account_type} <= 2;

	my $this_account_no = HSPC::WebDB::quote($so->{account_no});
	my $this_account_type = HSPC::WebDB::quote($so->{account_type});

	my $sql;

	if ($what eq 'STAFF') {
		$sql = qq {
			SELECT COUNT(*)
			FROM person p
			INNER JOIN person_account x
				ON x.person_id = p.person_id
				AND x.account_no = $this_account_no
				AND x.status != 255
		};
	}
	elsif ($what eq 'RESELLERS') {
		return undef unless $so->{account_type} == 0;
		$sql = q {
			SELECT COUNT(DISTINCT p.person_id)
			FROM person p
			INNER JOIN person_account x
				ON x.person_id = p.person_id
				AND x.status != 255
			INNER JOIN account ac
				ON ac.account_no = x.account_no
				AND ac.account_type = 2
		};
	}
	elsif ($what eq 'CUSTOMERS') {
		# Limit Reseller's visibility scope
		my $scope_constraint = '';
		$so->{account_type} == 0 or do {
			$scope_constraint = qq {
				AND (ac.vendor = $this_account_no OR ac.account_no = $this_account_no)
			}
		};
		$sql = qq {
			SELECT COUNT(DISTINCT p.person_id)
			FROM person p
			INNER JOIN person_account x
				ON x.person_id = p.person_id
				AND x.status != 255
			INNER JOIN account ac
				ON ac.account_no = x.account_no
				AND ac.account_type = 3
				$scope_constraint
		};
	}
	else {
		$sql = $so->{account_type} == 0 ?
			qq {
				SELECT COUNT(*)
				FROM person
			} :
			qq {
				SELECT COUNT(DISTINCT p.person_id)
				FROM person p, person_account x, account ac
				WHERE p.created_by_acct_no = $this_account_no OR
				(x.person_id = p.person_id AND ac.account_no = x.account_no AND x.status != 255
				AND (ac.vendor = $this_account_no OR ac.account_no = $this_account_no))
			};

	}
	return HSPC::WebDB::select_result($sql);
}


#### VE Persons

# => email
# => prefix
# => fname
# => mname
# => lname
# => suffix
# => ve_id
# => login
sub insert_ve_person {
	my $class = shift;
	my %h = ( DEBUG=>1,
	@_);

	my $person_info;
	$person_info->{email} = $h{email};
	$person_info->{name_title} = $h{prefix};
	$person_info->{name_first} = $h{fname};
	$person_info->{name_middle} = $h{mname};
	$person_info->{name_last} = $h{lname};
	$person_info->{name_suffix} = $h{suffix};

	my $so = $ENV{security_obj};
	HSPC::MT::Person::insert($person_info, $so->{account_no});
	warn "VE_ID=$h{ve_id},PERSON_ID=$h{person_id},LOGIN=$h{login}" if $h{DEBUG};
	HSPC::WebDB::select_run(q{INSERT INTO person_ve (ve_id, person_id, login_name) VALUES (?,LAST_INSERT_ID(),?)},$h{ve_id},$h{login});
}

# =>hash:
# 	ve_id =>
#	person_id =>
#	login =>
sub insert_ve_person_login {
	my $class = shift;
	my %h = ( DEBUG=>1,
	@_);
	warn "VE_ID=$h{ve_id},PERSON_ID=$h{person_id},LOGIN=$h{login}" if $h{DEBUG};
	HSPC::WebDB::select_run(q{INSERT INTO person_ve (ve_id, person_id, login_name) VALUES (?,?,?)},$h{ve_id},$h{person_id},$h{login});
}

# => ve_id
# => old_login
# => new_login
# => old_person_id
# => new_person_id
sub update_ve_login {
	my $class = shift;
	my %h = (
	@_);

	HSPC::WebDB::select_run(q{update person_ve set login_name = ?, person_id = ?
	where ve_id = ? and person_id = ?}, $h{new_login}, $h{new_person_id}, $h{ve_id}, $h{old_person_id});
}


# Returns list of staff members for this VE
sub get_list_person_ve {
	my ($class, $info, $ve_id) = @_;

	# retreive security object
	my $so = $ENV{security_obj};
	my $this_account_no = HSPC::WebDB::quote($so->{account_no});
	my $this_ve_id = HSPC::WebDB::quote($ve_id);

	my $tables = qq {
			person_ve pv,
			person p
			INNER JOIN person_account x
				ON x.person_id = p.person_id
				AND x.account_no = $this_account_no
				AND x.status != 255

	};

	HSPC::WebDB::select_data (
		request		=> $info,
		tables		=> $tables,
		where		=> q {	pv.ve_id = $this_ve_id },
		distinct	=> 1,
		gate_cols	=> {
			p			=> 'p.person_id',
			name		=> q {
							rtrim( concat(
								if(strcmp(p.prefix, ''), concat(p.prefix, ' '), ''),
								if(strcmp(p.fname,  ''), concat(p.fname,  ' '), ''),
								if(strcmp(p.mname,  ''), concat(p.mname,  ' '), ''),
								if(strcmp(p.lname,  ''), concat(p.lname,  ' '), ''),
								if(strcmp(p.suffix, ''), concat(p.suffix, ' '), '')
							) )
						   },
			email		=> 'p.email',
			login		=> 'pv.login',
		},
	);
}

# return array ve_id for person
sub get_person_ve_list {
	my $self = shift;
	my $person_id = shift;

	return HSPC::WebDB::select_rows(q{ select pve.ve_id
				from person_ve pve
				where pve.person_id = ?
				},$person_id);
}


# return person login
sub get_person_ve_login {
	my $self = shift;
	my $person_id = shift;
	my $ve_id = shift;

	return HSPC::WebDB::select_result(q{ select pve.login_name
				from person_ve pve
				where pve.person_id = ? and pve.ve_id = ?
				},$person_id, $ve_id);
}

# return person info by login and ve_id
sub get_ve_person_info {
        # there was code using WebDB with auth_asppassport table
        sw_die("Things are going really wrong. Ask baza@ or dkoles@ fro info");
}

sub visible {
	my ($person_id, $account_no) = @_;
	my $so = $ENV{security_obj};
	my $this_account_type = $so->account_type;

	if ($this_account_type == 0) {
		return 1;
	}
	elsif ($this_account_type == 3) {
		return $so->account_no == $account_no ? 1 : 0;
	}
	else {
		my $sql = qq {
			SELECT p.person_id
			FROM person p
			LEFT OUTER JOIN person_account x
				ON x.person_id = p.person_id
				AND x.status != 255
			LEFT OUTER JOIN account ac
				ON ac.account_no = x.account_no
			WHERE
				p.person_id = ? AND
				( ac.vendor = ? OR ac.account_no = ? OR p.created_by_acct_no = ? )
		};
		$sql .= ' AND  ac.account_no = ' . HSPC::WebDB::quote($account_no) if defined $account_no;
		my $a = $so->account_no;
		my $r = HSPC::WebDB::select_result($sql, $person_id, $a,  $a, $a);
		return $r == $person_id ? 1 : 0;
        };
}

1;

__END__
