package HSPC::SimpleActionLog;

use HSPC::MT::OM::Action;
use HSPC::MT::Core;
use HSPC::Error;
use HSPC::ErrorStack;
use HSPC::MT::Core::Constants qw(:DEFAULT);
use HSPC::Logger qw(sw_report_problem sw_log_warn sw_die);
use HSPC::Localization;

use strict;

sub new {
	my $class = shift;
	my %h = (
			 node_id => '',
			 node_name => '',
			 node_type => '',
			 node2_id => '',
			 node2_name => '',
			 node2_type => '',
			 module_id => '',
			 action_id => '',
			 problem_id => '',
			 @_
	);

	sw_die('Not enough params to call ActionLog') if(!$h{node_id} || !$h{node_type} || !$h{action_id});

	my $email = '';
	my $username = '';
	my $account_no = '';
	my $ip_address = '';

	if($ENV{taskman_obj}) { 
		my $taskman_args = $ENV{taskman_obj}->get_task_args(); 
		$ip_address = $taskman_args->{ip_address} unless ($ip_address); 
		$email = $taskman_args->{email} unless ($email); 
		$username = $taskman_args->{username} unless ($username); 
		$account_no = $taskman_args->{account_no} unless ($account_no); 
	} 
	else
	{
		my $sec = $ENV{security_obj};
		if ($sec && $sec->person_id) {
			my $person = HSPC::MT::Core->find_person(id=>$sec->person_id);
			if($person->id) {
				$email = $person->email;
				$username = join(' ', grep { $_ }
					($person->fname, $person->mname, $person->lname));
			}
		}
		$account_no = $sec ? $sec->account_no : '';
		$ip_address = $ENV{REMOTE_ADDR};
	}

	my $action = new HSPC::MT::OM::Action;
	
	my $self = {
			node_id => $h{node_id},
			node_name => $h{node_name},
			node_type => $h{node_type},
			node2_id => $h{node2_id},
			node2_name => $h{node2_name},
			node2_type => $h{node2_type},
			module_id => $h{module_id},
			action_id => $h{action_id},
			action => $action,
			problem_id => '',
			ip_address => $ip_address,
			account_no => $account_no,
			email => $email,
			username => $username,
	};
	bless $self, $class;

	return $self;
}

##sub DESTROY {
##	$self = shift;
##	
##}

#------------------------------- _log_start --------------------------------#
sub log_start {
	my $self = shift;

	my $action = $self->{action};

	$action->set(
			node_id => $self->{node_id},
			node_name => $self->{node_name},
			node_type => $self->{node_type},
			node2_id => $self->{node2_id},
			node2_name => $self->{node2_name},
			node2_type => $self->{node2_type},
			module_id => $self->{module_id},
			action_id => $self->{action_id},
			ip_address => $self->{ip_address},
			account_no => $self->{account_no},
			email => $self->{email},
			username => $self->{username},
	);
	$action->save();
}

##------------------------------- _log_finish -------------------------------#
##>> class
##=>> res = 0 success | 1 - error
##=>> comment default ''
sub log_finish {
	my $self = shift;
	my %h = (
			 res => '',
			 comment => '',
			 @_,
	);
	my $res = $h{res};
	my $comment = $h{comment};
	
	my $problem_id = $self->problem_id();
	my $action = $self->action;
	
	if($action) {
		my $errcode;
		if($res) {
			$errcode = last_error();
		}
		if($comment) {
			$action->comment($comment);
		}
		if($errcode) {
			my $error_obj = $ENV{error_obj};
			$action->result($error_obj->ext2normal_code($errcode));
			$action->error_code($errcode);

			$action->problem_id($problem_id);
		} else {
			$action->result(0); # success
		}
		$action->finished(-1);
		$action->save();
	}
}

sub action {
	my $self = shift;
	return $self->{action};
}

sub problem_id {
	my $self = shift;
	my %h = (
			 problem_id => '',
			 @_,
	);

	if($h{problem_id}) {
		$self->{problem_id} = $h{problem_id};
	} else {
		return $self->{problem_id};
	}
}

##function print error message and report about problem. If action executed as task this task is updating.
##>self
##=>problem_text
##=>error_message
##=>message_as_text
##<self->{problem_id}
sub report_problem {
	my $self = shift;
	my %h =	(
			 problem_text => '',
			 message_as_text => '',
			 error_message => '',
			 @_
	);

	my $text = $h{problem_text};
	my $message_as_text = $h{message_as_text};
	$message_as_text = 1 if (!defined($h{message_as_text}) && !defined($h{error_message}));
	my $error_message = $h{error_message};
	$error_message = $text if ($message_as_text);
	
	sw_log_warn($error_message) if (length($error_message));

	my $problem_id = sw_report_problem(text => $text);

	$self->problem_id(problem_id => $problem_id);
}

1;
