# -*- perl -*-
# HSPC::UISteps
#
# abstractclass for multistep UI-interface
#
# Author: Max Bazarov
# (c) 2001 SWsoft, All rights reserved


package HSPC::UISteps;
use strict;
use HSPC::Config;
use HSPC::Localization qw(:DEFAULT string string_lang);  

use HSPC::Logger qw(sw_die);
use HSPC::WebSystemLib;
use SWsoft::protect;	# Mark this module for encryption upon build


# =>  start_step
# < ref to HSPC::UISteps instance
sub new{
	my $self = {class=>shift};		
	bless ($self,$self->{class});		
	
	my %h=(@_);
	
	$self->{__start_step_code} = $h{start_step};	
	$self->{__was_inited} = 0;	
		
	return $self;
}

sub run{
	my $self = shift;
	
	# init self #
	unless($self->{__was_inited}){
		$self->init;
	}
	
	my $res;		
	
	# PROCESS STEP #
	if(defined $self->step_code){
		my $step_processing = $self->step_code;
		HSPC::Logger->debug('[CORE:UISTEPS] handle step '.$self->step_code);
		$res = $self->process_step(step_code=>$self->step_code);
			
		if(!$res || ref($res) ne 'HASH'){			
			die qq{ERROR: Undefined result from process_step_}.$self->step_code.qq{() in UISteps->run()};	
		}
		
		# process COMPONENT_RESULT #	
		if( $res->{STATUS} ){		
			# change step #			
			$self->step_code( $self->calc_step(step_code=>$self->step_code,step_inc=>$res->{step_inc}) ) if defined $res->{step_inc};
			return $res;
		}
	
		$self->init_steps() if defined $res->{reinit_before} || (! $self->step(step_code=>$self->step_code) );
	
		# CALC NEW STEP #						
		# set step_inc #				
		if(defined $res->{step_inc} ){
			my $new_step = $self->calc_step(step_code=>$self->step_code,step_inc=>$res->{step_inc});			
			if( $new_step eq 'was_last_step' ){
				my $res = $self->handle_finish();
				return $res if $res->{STATUS};
				$new_step = $self->step_code;
			}
			if( $new_step eq 'was_first_step' ){
				my $res = $self->handle_zerostep();
				return $res if $res->{STATUS};
				$new_step = $self->step_code;
			}
			$self->step_code( $new_step );
		# set defined step #				
		}elsif(defined( $res->{step_code} )){
			$self->step_code( $res->{step_code} );
		}else{
			my $c = "ERROR : wrong result from step processing, step_code=$step_processing, known step,keys : ";
			map {$c.=$_.'='.$res->{$_}} keys %$res;
			die $c;
		}
		# /CALC NEW STEP #
	}else{
		$self->init_steps();
		# calc first step #						
		$self->step_code( $self->start_step?$self->start_step:$self->calc_step(step_inc=>1) );		
	}	
	# /PROCESS STEP #
	
	# REINIT STEPS #		
	$self->init_steps() if defined $res->{reinit_after};	
		
	if(defined $self->step_code){
		# SHOW NEW STEP #		
		HSPC::Logger->debug('[CORE:UISTEPS] show step '.$self->step_code);
		## try to choose node corresponding to wizard step
		my $step_teaser = $self->step_code;
		if ($ENV{COMPREP}) {
			my $node = $ENV{COMPREP}->node;
			$ENV{COMPREP}->node(alias => "$node->{alias}_step_$step_teaser", choose => 1)
				if $node;
		}
		$res = $self->teaser_step(step_code => $step_teaser);		
	}else{
		$res = $self->teaser_undef_step();
	};					
	return $res;
}

# > ref to hash {code=>>,title=>>||title_id=>>,full_title_id=>||full_title=>,devscreen_id=>} or array of such hashes
sub add_step{
	my ($self,$steps) = @_;	
	die("HSPC::UISteps->add_step: no steps") unless defined $steps;
	
	$steps = [$steps] unless ref($steps) eq 'ARRAY';
	
	foreach my $step( @$steps ){
		die("HSPC::UISteps->add_step: no step code") unless defined $step->{code};
		die("HSPC::UISteps->add_step: no step title || title_id") unless (
			defined $step->{title}||defined $step->{title_id}
		);
		push @{$self->{__steps}},$step;
		if( defined $step->{title_id} ){
			$step->{title} = string($step->{title_id});
			$step->{title_id} = undef;
		}
		if( defined $step->{full_title_id} ){
			$step->{full_title} = string($step->{full_title_id});
			$step->{full_title_id} = undef;
		}
		$step->{full_title} = $step->{title} unless defined $step->{full_title};
		$self->{__steps_hash}->{ $step->{code} } = $step;
	}
}

sub init_steps{
	my $self = shift;
	
	# reset steps #
	$self->{__steps} = undef;
	$self->{__steps_hash} = undef;
	$self->{__steps_initing} = 1;
	
	# init steps
	$self->init_self_steps();
	
	# recalc step indexes #
	$self->_recalc_steps;	
	
	$self->{__steps_initing} = undef;
}

sub init_self_steps{
	my $self = shift;
}	

#  delete step from steps list
# =>> step
sub delete_step{
	my ($self,%h) = @_;	
	die unless defined $h{step};

	## search step index
	my $step_index=-1;
	my $step;
	my $steps = $self->{__steps};	
	for(my $index=0;$index<=$#$steps;$index++ ){
		my $step = $steps->[$index];
		next unless $step->{code} eq $h{step};
		$step_index = $index;
		last;
	}
	
	### check step index
	return undef unless $step_index>=0;

	##### delete step		
	splice( @{$self->{__steps}} , $step_index, 1 );	
		
	##### check object state
	if($self->{__steps_initing}){	
	}else{
		$self->_recalc_steps;
	}
}

#  change steps order, move step to new place
# =>> step
# =>> after
#   or 
# =>> before
sub move_step{
	my ($self,%h) = @_;	
	die("HSPC::UISteps->move_step: step is required.") unless defined $h{step};
	die("HSPC::UISteps->move_step: after or before is required.") unless (
		defined $h{after}||defined $h{before}
	);	
	
	# remove req. step from list #
	my $step = $self->step( step_code=>$h{step} );
	my @steps = grep{$_->{code} ne $h{step}} @{$self->{__steps}};
	die if $#steps==$#{$self->{__steps}};				
			
	if(defined $h{after}){
		for(my $index=0;$index<=$#steps;$index++){		
			# seek new place #		
			if($steps[$index]->{code} eq $h{after}){				
				#splice(@steps,$index,0,$step);
				splice(@steps,$index,1,$steps[$index],$step);
				last;
			}
		}
	}elsif(defined $h{before}){
		for(my $index=0;$index<=$#steps;$index++){		
			# seek new place #		
			if($steps[$index]->{code} eq $h{before}){
				splice(@steps,$index,1,$step,$steps[$index]);
				last;
			}
		}
	}else{
		die;
	}
	
	
	$self->{__steps} = \@steps;
	
	if($self->{__steps_initing}){	
	}else{
		$self->_recalc_steps;
	}
}

# =>> step_code
sub process_step{
	my ($self,%h) = @_;
	my $step = $self->step(step_code=>$h{step_code});
	
	# process step #	
	die unless defined $h{step_code} && $h{step_code} ne '';
		
	my $res;		
	my $cmd = '$res = $self->process_step_'.($step->{call_code}||$h{step_code}).'()';
	eval $cmd;	
	if (not defined $res) {		
		sw_die('can`t eval cmd='.$cmd.' error='.$@);
	}
    
	return $res;			
}

# =>> step_code
sub teaser_step{
	my ($self,%h) = @_;
	my $step = $self->step(step_code=>$h{step_code});
	
	# process step #	
	sw_die('argument step_code=> is undefined or empty') unless defined $h{step_code} && $h{step_code} ne '';
				
	my $res;	
	my $cmd = '$res = $self->teaser_step_'.($step->{call_code}||$h{step_code}).'();';
	eval $cmd;
	if (!$res || $@) {
		exit if $ENV{layout} && $ENV{layout}->{status} ne 'OK';
		sw_die('ERROR: '.$@.' , COMMAND='.$cmd)
			unless $ENV{"_redirect_ok"};
    }		    
    return $res;
}

sub teaser_undef_step{
	my $self = shift;
	sw_die('ERROR : You must redefine UISteps->teaser_undef_step() method');
}

# => new start_step code
# < current start_code
sub start_step{
	my ($self,$v) = @_;
	$self->{__start_step_code} = $v if defined $v;
	return $self->{__start_step_code};
}

# > new step_code
# < current step code
sub step_code{
	my ($self,$v) = @_;
	$self->{__step_code} = $v if defined $v;	
	return $self->{__step_code};
}


# < current step devscreen_id
sub step_devscreen_id{
	my ($self) = @_;	
	return $self->step->{devscreen_id};
}

# < ref to steps array [{code=>'',title=>'',index=>},..]
sub steps{
	my $self = shift;
	return $self->{__steps};
}

# =>> step_code
#  or
# =>> step_index
#  or
# return current step
################
# => allow_null : allow to return undef step
#  or return current step
# < ref to hash with step_info
sub step{
	my ($self,%h) = @_;	
	
	if(defined $h{step_code}){
		my $step = $self->{__steps_hash}->{$h{step_code}};
		unless( $step || $h{allow_null} ){
			$ENV{system_obj}->warn_calls_stack();
			die 'ERROR: can`t find step for step_code='.$h{step_code} ;
		}
		return $step;		
	}elsif(defined $h{step_index}){
		my $step = $self->steps->[$h{step_index}];			
			unless( $step || $h{allow_null} ){
				$ENV{system_obj}->warn_calls_stack();
				die 'ERROR: can`t find step for step_index='.$h{istep_ndex} ;
			}
		return $step;		
	}else{		
		my $step_code = $self->step_code;
		if( not defined $step_code ){
			$ENV{system_obj}->warn_calls_stack();
			die 'ERROR : step_code undefined in step()';	
		}
		return $self->step(step_code=>$step_code);
	}
}

####################
## => allow_null : allow to return undef step
##  or return current step
## < ref to hash with step_info
sub last_step{
	my ($self,%h) = @_;	
	
	my $step = $self->steps->[ $#{$self->steps} ];			
	unless( $step || $h{allow_null} ){
		$ENV{system_obj}->warn_calls_stack();
		die 'ERROR: can`t find last step';
	}
	return $step;		
}


# =>  step_code
# =>> step_inc
# < new step code / -1 = it was a last step
sub calc_step{	
	my ($self,%h) = @_;	
	my $index=0;
	
	if(!$h{step_code}){
		# GOTO FIRST STEP #
		return $self->step(step_index=>0)->{code};		
	}
	
	my $steps = $self->steps;
	foreach( @$steps ){		
		if($_->{code} eq $h{step_code}){
			$index+=$h{step_inc};
			return 'was_first_step' if $index<0; # OOPs...try to go on step with index=-1
			my $step = $steps->[$index];
			return 'was_last_step' unless $step;			
			return $step->{code};
		}
		$index++;
	}
	sw_die("ERROR: calc_step() can`t find step '$h{step_code}'");
}

sub handle_finish{
	my $self = shift;
	sw_die('ERROR : your must redefine '.__PACKAGE__.'->handle_finish() method (class is '.ref($self));
}

sub handle_zerostep{
	my $self = shift;
	die 'ERROR : your must redefine '.__PACKAGE__.'->handle_zerostep() method';
}

sub init{
	my $self = shift;
	
	$self->{__was_inited} = 1;
}

sub _recalc_steps{
	my $self = shift;
	
	# recalc step indexes #
	my $steps = $self->steps;
	sw_die("Steps not inited") unless ( ref($steps) eq 'ARRAY' );
	for(my $i=0;$i<= $#{$self->steps};$i++){
		$steps->[$i]->{index} = $i;
	}	
}


sub get_next_step_code {
	my $self = shift;
	my %h = (
		step_code => undef,
		@_
	);

	my $steps = $self->steps;
	my $current_step_code = $h{step_code} || $self->step_code;

	sw_die("[UISteps] step_code must be defined!")
		unless( $current_step_code );

	for( my $i = 0; $i < $#{ $steps }; $i++ ) {
		if(
			$steps->[$i]->{'code'} eq $current_step_code &&
			defined( $steps->[$i+1] )
		) {
			return $steps->[$i+1]->{'code'};
		}
	}

	return undef;
}


sub is_step_exists {
	my $self = shift;
	my %h = (
		step_code => undef,
		@_
	);

	sw_die("[UISteps] step_code must be defined!")
		unless( $h{step_code} );

	my $steps = $self->steps;

	return scalar( grep { $_->{'code'} eq $h{'step_code'} } @{$steps} );
}


1;

__END__

# protected attribites #

	__start_step_code : 
	__steps : ref to array [{code=>'',title=>'',title_id=>'',index=>N},...]
	__steps_hash : ref to hash {code1=>{code=>'',title=>'',title_id=>'',index=>N},code2=>{}..}
	__step_code : current step code	
	__was_inited : order was initted ( init() )		

# /protected attribites #

Child _CAN_ redefine next virtual methods:
	teaser_undef_step(	



Child _MUST_ redefine next virtual methods:
	init_self_steps()	
	handle_finish()
	handle_zerostep()
	
