# -*- perl -*-
# HSPC::UIStepsSession
#
# abstract class for multistep UI-interface in Office appliciations
#
# Author: Max Bazarov
# (c) 2001 SWsoft, All rights reserved


package HSPC::UIStepsPage;

use HSPC::UIStepsSession;
use vars qw{ @ISA };
@ISA = qw{ HSPC::UIStepsSession };

use strict;
use HSPC::Config;

use HSPC::Logger qw(sw_die);
use HSPC::WebSystemLib;
use HSPC::Localization qw(:DEFAULT string string_lang);  
use SWsoft::protect;	# Mark this module for encryption upon build


# =>> page
sub run{
	my ($self,%h) = @_;
	
	die unless $h{page};
	$self->page($h{page});
	
	my $res = $self->SUPER::run(%h);	
	
	# drop page #
	$self->page(undef);	
	
	return $res;
}

# =>! step_code
# < {COMPONENT_STATUS....,step_code=>,step_inc=>}
sub process_step{
	my ($self,%h) = @_;
		
	return undef unless $h{step_code};	
		
	my $res = $self->_process_form_before();
	$res =  $self->SUPER::process_step(%h)  unless defined $res;	
	$res = $self->_process_form_after($res) if(not defined $res) || $res->{STATUS} eq 'PROCESS';
	
	return $res;	
}

# =>> step_code
sub teaser_step{
	my ($self,%h) = @_;
	
	my $res =  $self->SUPER::teaser_step(%h);		
    
    if($res->{STATUS} eq 'OK' && (not defined $res->{CONTENT})){        	
    	return {STATUS=>'OK',CONTENT=>$self->page->get_content()};
    }        
	return $res;	
}

# >= string || string_id
sub add_error{
	my ($self,%h) = @_;

	$h{string} = string($h{string_id}) if ( $h{string_id} );
	
	$self->page->add_error(string=>$h{string});
}


## >> self
## < true/false
sub has_errors {
	my $self = shift;
	return $self->page->has_errors();
}##/has_errors


# > page
sub page{
	my $self = shift;
	
	if(@_){				
		$self->{__page} = shift;
	}		
	return $self->{__page};
}

# => object
# => disable_header
# => is_transparent
# => form_cols
# => is_multipart
##=> url_ext
sub form_open{
	my ($self,%h) = @_;
				
	my $step = $self->step;
	die "ERROR : can`t find step '$self->step_code'" unless $step;
	my $page = $self->page;
	
	$page->edit_open(
		title_id=>$step->{title_id},
		title=>$step->{title},
		form_url=>$self->process_url,
		object=>$h{object},
		disable_header=>$h{disable_header},
		form_width => $h{form_width},		
		is_transparent => $h{is_transparent}, 
		form_cols => $h{form_cols},
		is_multipart=>$h{is_multipart},
		url_ext=>$h{url_ext}
	);
	$page->edit_view_hidden(view_name=>'__step',value=>$step->{code});	
}

# =>id : id>0 = show buttons for existed object
# INTERFACE FOR NEW OBJECT #
# =>new_btns 
#		=> 'wc_back_next' =  show "Back|Next" buttons
#		=> 'wc_back_finish' =  show "Back|Finish" buttons
#		=> 'back_next' =  show "Cancel Back|Next" buttons
#		=> 'back_save' =  show "Cancel Back|Save" buttons
#		=> 'back_preview_save' =  show "Cancel Back|Preview|Save" buttons
#		=> 'cancel_back_next_finish' = show "Cancel Back|Next|Finish" buttons
#		=> 'cancel_next_finish' = show "Cancel Next|Finish" buttons
#		=> 'cancel_back_finish' = show "Cancel Back|Finish" buttons
#		=> 'cancel' = show "Cancel" button only
# /INTERFACE FOR NEW SKIN #
# INTERFACE FOR EXISTED OBJECT #
# => edit_btns : 			
#		=>'save' = show "Cancel Save" buttons
#		=>'preview_save' = show "Cancel Preview|Save" buttons
#		=>'delete_save' = show "Cancel Delete|Save" buttons
#		=>'delete_preview_save' = show "Cancel Delete|Preview|Save" buttons
#		=>'back_save' = show "Cancel Back|Save" buttons
#		=>'back_next' = show "Cancel Back|Next" buttons
#		=>'preview_next' = show "Cancel Preview|Next" buttons
#		=>'preview_back_save' = show "Cancel Preview|Back|Save" buttons
#		=>'cancel_back_next_finish' = show "Cancel Back|Next|Finish" buttons
#		=>'cancel_next_finish' = show "Cancel Next|Finish" buttons
#		=>'cancel_back_finish' = show "Cancel Back|Finish" buttons
# /INTERFACE FOR EXISTED OBJECT #

sub form_close{
	my ($self,%h) = @_;
	my $page = $self->page;
	
	$page->edit_view_hidden(view_name=>'__step_obj',value=>$h{id});
	
	return undef unless defined $h{new_btns} || defined $h{edit_btns};
	
	if(!$h{id}){
		my $step = $self->step();
		my $btns = [];
		if($h{new_btns} eq 'back_next'){
			push @$btns, {title=>string('back'),name=>'btn_back',
			              type=>'submit',class=>'SWIButtonX'} if $step->{index}>0;
			$page->edit_close(			
				upd_id=>'next',			
				cancel_id=>'cancel',
				btns=>$btns
			);
		}elsif($h{new_btns} eq 'back_save'){
			push @$btns, {title=>string('back'),name=>'btn_back',
			              type=>'submit',class=>'SWIButtonX'} if $step->{index}>0;
			$page->edit_close(			
				upd_id=>'save',			
				cancel_id=>'cancel',
				btns=>$btns
			);
		}elsif($h{new_btns} eq 'back_preview_save'){
			push @$btns, {title=>string('back'),name=>'btn_back',
			              type=>'submit',class=>'SWIButtonX'} if $step->{index}>0;
			push @$btns, {title=>string('preview'),name=>'btn_preview',check_form=>1,type=>'submit'};
			$page->edit_close(			
				upd_id=>'save',			
				cancel_id=>'cancel',
				btns=>$btns
			);
		}elsif($h{new_btns} eq 'cancel_back_next_finish'){
			push @$btns, {title=>string('back'),name=>'btn_back',
			              type=>'submit',class=>'SWIButtonX'} if $step->{index}>0;
			push @$btns,{title=>string('next'),name=>'btn_next',check_form=>1,class=>'SWButtonX',type=>'submit'};
			$page->edit_close(
				upd_id=>'finish',			
				cancel_id=>'cancel',
				btns=>$btns
			);
		}elsif($h{new_btns} eq 'cancel_back_finish'){
			push @$btns, {title=>string('back'),name=>'btn_back',
			              type=>'submit',class=>'SWIButtonX'} if $step->{index}>0;	
			$page->edit_close(
				upd_id=>'finish',
				cancel_id=>'cancel',
				btns=>$btns,
			);		
		}elsif($h{new_btns} eq 'cancel_next_finish'){
			$page->edit_close(			
				upd_id=>'finish',			
				cancel_id=>'cancel',
				btns=>[{title=>string('next'),name=>'btn_next',check_form=>1,class=>'SWButtonX',type=>'submit'}]
			);			
		}elsif($h{new_btns} eq 'wc_back_next'){
			push @$btns, {title=>string('back'),name=>'btn_back',
			              type=>'submit',class=>'SWIButtonX'} if $step->{index}>0;
			$page->edit_close(			
				upd_id=>'next',			
				cancel_id=>0,
				btns=>$btns
			);		
		}elsif($h{new_btns} eq 'wc_back_finish'){
			push @$btns, {title=>string('back'),name=>'btn_back',
			              type=>'submit',class=>'SWIButtonX'} if $step->{index}>0;
			$page->edit_close(
				upd_id=>'finish',			
				cancel_id=>0,
				btns=>$btns
			);
		}elsif($h{new_btns} eq 'cancel'){
			$page->edit_close(			
				upd_id=>0,			
				cancel_id=>'cancel',
			);
		}else{
			sw_die('[UISTEPSPAGE] uknown new buttons configuration, code='.$h{new_btns}.', step='.$self->step_code);			
		}
	}elsif($h{edit_btns} eq 'save'){
		$page->edit_close(			
			upd_id=>'save',
			cancel_id=>'cancel'			
		);
	}elsif($h{edit_btns} eq 'delete_save'){		
		$page->edit_close(	
			btn_del=>1,		
			upd_id=>'save',
			del_id=>'delete',
			cancel_id=>'cancel'			
		);
	}elsif($h{edit_btns} eq 'delete_preview_save'){		
		$page->edit_close(	
			btn_del=>1,		
			upd_id=>'save',
			del_id=>'delete',
			cancel_id=>'cancel',
			btns=>[{title=>string('preview'),name=>'btn_preview',check_form=>1,type=>'submit'}]
		);
	}elsif($h{edit_btns} eq 'back_save'){
		$page->edit_close(			
			upd_id=>'save',			
			cancel_id=>'cancel',
			btns=>[{title=>string('back'),name=>'btn_back',type=>'submit',class=>'SWIButtonX'}]
		);
	}elsif($h{edit_btns} eq 'back_next'){
		$page->edit_close(			
			upd_id=>'0',			
			cancel_id=>'cancel',
			btns=>[{title=>string('back'),name=>'btn_back',type=>'submit',class=>'SWIButtonX'},{title=>string('next'),name=>'btn_next',check_form=>1,class=>'SWButtonX',type=>'submit'}]
		);		
	}elsif($h{edit_btns} eq 'edit_save'){
		$page->edit_close(			
			upd_id=>'save',			
			cancel_id=>'cancel',
			btns=>[{title=>string('edit'),name=>'btn_back',type=>'submit',class=>'SWIButtonX'}]
		);		
	}elsif($h{edit_btns} eq 'preview_next'){
		$page->edit_close(			
			upd_id=>'0',
			cancel_id=>'cancel',
			btns=>[{title=>string('preview'),name=>'btn_preview',check_form=>1,type=>'submit'},{title=>string('next'),name=>'btn_next',check_form=>1,class=>'SWButtonX',type=>'submit'}]
		);
	}elsif($h{edit_btns} eq 'preview_back_save'){
			$page->edit_close(			
			upd_id=>'save',			
			cancel_id=>'cancel',
			btns=>[{title=>string('preview'),name=>'btn_preview',check_form=>1,type=>'submit'},{title=>string('back'),name=>'btn_back',check_form=>1,type=>'submit',class=>'SWIButtonX'}]
		);
	}elsif($h{edit_btns} eq 'cancel_back_next_finish'){
		$page->edit_close(			
			upd_id=>'finish',			
			cancel_id=>'cancel',
			btns=>[{title=>string('back'),name=>'btn_back',check_form=>1,type=>'submit',class=>'SWIButtonX'},{title=>string('next'),name=>'btn_next',check_form=>1,class=>'SWButtonX',type=>'submit'}]
		);
	}elsif($h{edit_btns} eq 'cancel_back_finish'){
		$page->edit_close(			
			upd_id=>'finish',			
			cancel_id=>'cancel',
			btns=>[{title=>string('back'),name=>'btn_back',check_form=>1,type=>'submit',class=>'SWIButtonX'}]
		);		
	}elsif($h{edit_btns} eq 'cancel_next_finish'){
		$page->edit_close(			
			upd_id=>'finish',			
			cancel_id=>'cancel',
			btns=>[{title=>string('next'),name=>'btn_next',check_form=>1,class=>'SWButtonX',type=>'submit'}]
		);	
	}elsif($h{edit_btns} eq 'preview_save'){
			$page->edit_close(			
			upd_id=>'save',			
			cancel_id=>'cancel',
			btns=>[{title=>string('preview'),name=>'btn_preview',check_form=>1,type=>'submit'}]
		);
	}else{
		sw_die(
			'ERROR : uknown edit buttons configuration, code='.$h{edit_btns}
			.', step='.$self->step_code
		);
	}	
}

# < process_form_url 
sub process_url{
	my $self = shift;
	return $self->page->get_browse_url.'/act,p';
}


sub _process_form_before{
	my $self = shift;
	my $step_code = $self->step_code;
	
	return undef unless $step_code;
	return {step_inc=>0} unless sw_argparam('__step') eq $step_code;
	return $self->handle_cancel() if sw_argparam('item_cancel');
	return $self->handle_delete() if sw_argparam('item_delete');
	return undef;
}

### return 1 if was pressed save,preview r next buttons
## >> self
## < result
sub is_next{
	my $self = shift;
	return sw_argparam('btn_next')||sw_argparam('btn_preview')
		||sw_argparam('item_submit');
}

## >> self
## >> ref to process step hash result
sub _process_form_after{
	my ($self,$h) = @_;
	$h->{STATUS} = undef;
	
	return {%$h,step_inc=>-1} if sw_argparam('btn_back');
	
	if(!sw_argparam('__step_obj')){		
		return {%$h,step_inc=>1} if sw_argparam('item_submit');
	}else{			
		return {%$h,step_inc=>1} if sw_argparam('btn_next');
		return {%$h,step_code=>'preview'} if sw_argparam('btn_preview');
		return $self->handle_save() if sw_argparam('item_submit');
	}
	return { %$h,step_inc=>0 };	
}

sub handle_cancel{
	my $self = shift;
	die 'ERROR : your must redefine '.__PACKAGE__.'->handle_cancel() method';
}
sub handle_save{
	my $self = shift;
	die 'ERROR : your must redefine '.__PACKAGE__.'->handle_save() method';
}

1;

__END__

# public attribites #
	free_session : 
# /public attribites #	

# protected attribites #
	
	__page : ref to HSPC::WebPage instance

# /protected attribites #

Child _MUST_ redefine next virtual methods:
	
	sub handle_cancel()
	sub	handle_save()
	sub handle_delete() # if you have 'delete' buttons

Child _CAN_ redefine next virtual methods:

	sub process_url()
	sub _process_form_before()
	sub _process_form_after()	

	
