# -*- perl -*-
# HSPC::UIStepsSession
#
# abstract class for multistep UI-interface in Office appliciations
#
# Author: Max Bazarov
# (c) 2001 SWsoft, All rights reserved


package HSPC::UIStepsSession;

use HSPC::UISteps;
use vars qw{ @ISA };
@ISA = qw{ HSPC::UISteps };

use strict;
use Storable;
use HSPC::Config;

use HSPC::Logger qw(sw_die sw_log_debug sw_log_error);
use HSPC::WebSystemLib;
use HSPC::Localization qw(:DEFAULT string string_lang);  
use SWsoft::protect;	# Mark this module for encryption upon build

use constant STORE_PATH => '/var/opt/hspc-data/HSPC/UIStepsSession/objects';


# >   class
# =>  page
# =>> session_code
#   or
# =>=  steps : ref to HSPC::UISteps instance
# < COMPONENT_RESULT or undef, undef = no object in session
sub teaser_run{
	my ($class,%h)=@_;	
	
	sw_die('session_code=> undefined ') unless defined $h{session_code};
		
	my $self;
	
	if($h{steps}){
		$self = $h{steps};
		$self->store_in_env;
	}else{		
		$self = $class->restore_object(%h);
		unless(defined $self){
			return $class->_catch_no_object_in_session(%h);
		}
	}
	sw_die('can`t restore object') unless $self;	
		
	$self->{__session_code} = $h{session_code};
		
	my $res = $self->run(%h);
	$self->store_in_session;
	return $res;	
}

# >>  class 
# =>> session_code
# < COMPONENT_RESULT
sub _catch_no_object_in_session{
	my ($class,%h)=@_;
	
	unless(defined $h{page}){
		sw_die("[CORE:UIStepsSession] (ERROR) no object [$h{session_code}] in session");
	}
	
	return $h{page}->redirect('',1);
}

# >> class
# =>> session_code
# < ref to object
#     or 
#  undef  : not object in session
sub restore_object{
	my ($class,%h) = @_;
	sw_die('session_code=> argument was not defined') unless $h{session_code};
	my $obj = $ENV{'SESSION_'.$h{session_code}};
	
	return $obj if defined $obj;
	
	######### restore data
	$obj = $ENV{system_obj}->restore_data( key=>$h{session_code} );
	
	######### check data existing
	unless( $obj ){
		sw_log_debug('[CORE:UIStepsSession] (WARNING) session object {'.$h{session_code}.'} is undefined');
		return undef;
	}
	
	# REQUIRE CHILD CLASS #
	my $res;
	my $cmd = 'require '.ref($obj).';';
	eval $cmd;
	if( $@ ){
		sw_log_error(
			qq~ERROR : UIStepsSession->restore_object
			try to execute [$cmd]			
			$@
			ERROR : UIStepsSession->restore_object
		~);
		die 'ERROR: look hspc-log';			
	};
	$obj->restore_from_session;
	#
	
	$obj->store_in_env;
	
	return $obj;
}
##################################################################

# =>  start_step
sub new{
	my ($class,%h) = @_;		
	
	my $self  = $class->SUPER::new(%h);
		
	return $self;
}

sub store_in_env{
	my $self = shift;
	
	$ENV{'SESSION_'.$self->{__session_code}} = $self;
};

sub store_in_session{
	my $self = shift;		
	
	$self->{stored_total}++;	
	
	$ENV{system_obj}->store_data( key=>$self->{__session_code},
		data=>$self->{free_session}?undef:$self );		
}

sub restore_from_session{
	my $self = shift;	
}

sub session_code{
	my $self = shift;
	return $self->{__session_code};	
}

1;

__END__

# public attribites #
	free_session : 
	stored_total : how must times this object was stored in session	
# /public attribites #	

# protected attribites #
	__session_code:
# /protected attribites #

# methods #

	virtual sub restore_from_session();
	virtual sub store_in_session();

# /methods #

Child _MUST_ redefine next virtual methods:



	
