package HSPC::WebElements;

use strict;
use HSPC::Localization qw(:DEFAULT string string_lang);  
use HSPC::Config;
use HSPC::DataCheck;

use HSPC::Util qw(sw_find_custom_file);
use HSPC::Logger qw(sw_die);
use HSPC::MT::Core::Constants;
use HSPC::MT::Core::Web;
use HSPC::MT::Core;
use HSPC::WebSystemLib;
use HSPC::ErrorCodes;
use POSIX qw(ceil);
use SWsoft::protect;	# Mark this module for encryption upon build
use HSPC::MT::UM::TS;
use HSPC::PluginHelper::Base;

use Data::Dumper;

# > class
# > input {i,url,title_res_id,total_items,items_onpage,tq,hide_qp,hide_pc,url_index}
# < results
sub quickpreview_bar{	
	my $class=shift;
	my $i=shift;	
	
	return undef if $i->{hide_qp} && ($i->{hide_pc} || $i->{total_items}<=20);
	
	# quick_preview_bar #		
	my ($url_tq,$url_pc) = ($i->{url},$i->{url});
	$url_tq.='/pc,'.$i->{items_onpage} if $i->{items_onpage}!=20;
	$url_pc.='/tq,'.$i->{tq} if $i->{tq} ne 'to';
	
	my $pcs=[];
	push @$pcs,['20',20,$i->{items_onpage}==20] if $i->{total_items}>20;
	push @$pcs,['40',40,$i->{items_onpage}==40] if $i->{total_items}>21;
	push @$pcs,['80',80,$i->{items_onpage}==80] if $i->{total_items}>41;
		
	my $h={i=>$i->{i},						
		url_index=>$i->{url_index},
		hide_qp=>$i->{hide_qp},
		hide_pc=>$i->{hide_pc},
		url_tq=>$url_tq,
		url_pc=>$url_pc,
		pc=>$i->{items_onpage},
		pcs=>$pcs,
		tq=>$i->{tq},
		title_res_id=>$i->{title_res_id},
		total_items=>$i->{total_items},
	};		
	#
	return sw_web_parse_template($class,'qp_bar.tmpl',$h);	
}

# > class
# > input {i,url,title_res_id,current_item,total_items,items_onpage,class_app,hide_pc}
# < results
sub paging{
	my $class=shift;
	my $i=shift;	
			
	#
	my ($pages_onpage,$last_page,$from_page,$to_page)=(10,0,0);
	my $current_index = $i->{current_item};
	my $total_items = $i->{total_items};
	my $pc=$i->{items_onpage};
					
	if($current_index > $total_items){$current_index = $total_items};
	$last_page = int($total_items/$pc);
	if( ($last_page * $pc) < $total_items){$last_page++;}				
	$from_page = int($current_index / $pc);
	if($from_page<1){ $from_page=1 };			
	$from_page-=4;		
	if($from_page<1){ $from_page=1 };
	if ($from_page>1){
		$to_page = $from_page + $pages_onpage -1 ;	
		if( ($to_page*$pc) > $total_items){			
			$to_page = $last_page;
			$from_page = $to_page - $pages_onpage+1;
			if($from_page<=1){$from_page=1;};
		}
	}	
	$to_page = $from_page + $pages_onpage -1;		
	
	my $onpage_last = $current_index+$pc-1;	
	if($onpage_last>$total_items){ $onpage_last = $total_items };

	my $item_index = ($from_page-1) * $pc+1;				
	my $pages=[];
	for(my $page=$from_page;$page<=$to_page;$page++){		
		push(@$pages,[$i->{url}.$item_index,$page]);		
		#
		$item_index+=$pc;
		if($item_index>$total_items){ 	last;	}				
	}
	#
	my ($firstpage_url,$prevpage_url,$nextpage_url,$lastpage_url)=(undef,undef,undef,undef);
	if($current_index>1){
		$firstpage_url = $i->{url}.1;
		$prevpage_url = $i->{url}.($current_index-$pc);
	}
	if($onpage_last<$total_items){
		$nextpage_url = $i->{url}.($current_index+$pc);	
		$lastpage_url = $i->{url}.(($last_page-1)*$pc+1);
	}				
	#
	#### PC part
	my $url_pc = $i->{url_pc};
	my $pcs=[];
	my $up_pc = $ENV{HSPC_SYSTEM}->upref_pc;
		
	push @$pcs,['20',20,$i->{items_onpage}==20] if $i->{total_items}>20;
	push @$pcs,['40',40,$i->{items_onpage}==40 || ($up_pc>40 && $i->{total_items}<=40)] if $i->{total_items}>20;
	push @$pcs,['80',80,$i->{items_onpage}==80] if $i->{total_items}>40;

	
	#
	my $item_from_to=string('item_from_to');
	$item_from_to =~ s/%items%/string($i->{title_res_id})/sge;
	$item_from_to =~ s/%from%/$current_index/sg;	
	$item_from_to =~ s/%to%/$onpage_last/sg;
	$item_from_to =~ s/%count%/$i->{total_items}/sg;		
	
	my $item_per_page = string('item_per_page');
	$item_per_page =~ s/%items%/string($i->{title_res_id})/sge;
	
	my $h={
		i	=> $i->{i},
		#class_app=>$i->{class_app},
		item_from_to	=> $item_from_to,
		item_per_page	=> $item_per_page,
		str_first	=> string('first_lc'),
		str_last	=> string('last_lc'),
		str_prev	=> string('prev_lc'),
		str_next	=> string('next_lc'),
		str_col_vis	=> string('column_vis_sttngs'),
		str_exp_xls	=> string('export_to_excel'),
		title_res_id=> $i->{title_res_id},
		onpage_first=> $current_index,
		onpage_last	=> $onpage_last,
		current_page=> int($current_index/$pc)+1,
		total_items	=> $i->{total_items},
		prevpage_url=> $prevpage_url,
		nextpage_url=> $nextpage_url,
		firstpage_url	=> $firstpage_url,
		lastpage_url	=> $lastpage_url,
		pages		=> $pages,
		a	=> "$from_page-$to_page",
		#### pc part
		hide_pc	=> $i->{hide_pc},
		url_pc	=> $url_pc,
		pc		=> $i->{items_onpage},
		pcs		=> $pcs,
		hide_columns_custom	=> $i->{hide_columns_custom},
	};		
	#
	return sw_web_parse_template($class,'paging.tmpl',$h);	
}


# > class
# => pages, ref to array of hashes or undef
#	    caption=> Text string with Page Title, f.e. "General Info"
#          OR 
#	    caption_id=> Text string with i18n ID for page title
#       page=> Page Code, uses for current page determinating and make URL for this page, f.e. 'gen'
#       url=> Full URL for this page
#       ext_url=> Ext. URL for this page, will be added to url+page
#		target=> url target
# => url, base url for all tabs, for example :  '/office/news/a,e/id,12/tab,'
# => limit, tab limit in one line (use default if not defined)
# => cur_page : current page code
sub tab{
	my $class = shift;
	my %h = (@_);	
					
	if(1==3){
		my $limit_chars = 75;
		
		my @tabs;
		my $tab;
		my $tab_length;		
		my $curr_tab;
		foreach my $page (@{$h{pages}}){
			# convert i18n-ID into real string #
			if(defined $page->{caption_id}){
				$page->{caption} = string($page->{caption_id});
				$page->{caption_id} = undef;				
			}
			# inc current tab page #
			$tab_length+=length( $page->{caption} );
			if($tab_length>$limit_chars){
				push @tabs,$tab;
				$tab=undef;
				$tab_length=0;
			}
			# check for current page #
			if($page->{page} eq $h{cur_page}){
				$curr_tab = $#tabs+1;				
			}
			push @$tab,$page;
		}
		if($tab){			
			push @tabs,$tab;
		}
		
		# move tab with current page to bottom #
		if($curr_tab!=$#tabs){
			my $tab = $tabs[$curr_tab];
			splice(@tabs,$curr_tab,1,);
			push @tabs,$tab;
		}
		
		$h{pages} = \@tabs;
	}
	
	if(1==1){
		# old  #
		
		my $tabs;		
		my $limit = ($h{limit}) ? $h{limit} : 7;
		
		## skip undefined items
		my @pages = grep{defined $_} @{ $h{pages} };
		$h{pages} =  \@pages;
		
		my $total_pages = $#{$h{pages}}+1;
		my $posted_pages=0;
		
		if( $total_pages == 0 ){
			$h{pages} = undef;
		}elsif($total_pages<=$limit){
			$h{pages} = [$h{pages}];
		}else{
			my $total_tabs = ceil( $total_pages / $limit );			
			if($total_tabs==0){
				$tabs = [$h{pages}];
			}else{
				my $tab_pages = $total_pages / $total_tabs;						
				my $curr_tab;
				for(my $tab_index=0;$tab_index<$total_tabs;$tab_index++){
					my $first_page = ceil($tab_index*$tab_pages);
					my $last_page = ceil($first_page + $tab_pages-1);					
					$last_page = $total_pages-1 if $last_page >= $total_pages;					
					#
					my $pages;
					for(;$first_page<=$last_page;$first_page++){					
						push @$pages,$h{pages}->[ $first_page ];
						$posted_pages++;
						if($h{pages}->[ $first_page ]->{page} eq $h{cur_page}){
							$curr_tab = $tab_index;
						}
						#last;						
					}
					push @$tabs,$pages;												
					#last;
				}
				## move tab with current page to bottom ##
				if($curr_tab!=$total_tabs){
					my $tab = $tabs->[$curr_tab];
					splice(@$tabs,$curr_tab,1,);
					push @$tabs,$tab;
				}
			}
							
			$h{pages} = $tabs;					
			die 'CALL BAZA !' if $posted_pages!=$total_pages;
		}				
	}
	
	return sw_web_parse_template($class,'tab.tmpl',\%h);	
};

# > class
###### TYPE_API: strings ####
# =>> string_id 
#   ( OR/AND ) 
# =>> string
# =>  is_warning (1,0)
###### TYPE_API : errors ####
# => error_id
######################
# < string_results
#
# Allowed string_id are:
#   - localized string ID like "error_message"
#   - list of localized string IDs like "error_message1,error_message2"	
#   - error code like "12345" (see HSPC::ErrorCodes)
#   - list of error codes like "1234,789"
#
sub status_message
{
	my $class = shift;
	my %h=(@_);
	my @statuses;
	
	return undef unless $h{string_id} || $h{string} || $h{error_id};	
	
	if(defined $h{error_id}){		
		$h{is_warning}=1;
		foreach( split (',', $h{error_id}) ){
			push @statuses,_error_id_2_str($_);
		}		
	}else{		
		push @statuses,$h{string} if defined $h{string};
		
		if(defined $h{string_id}){			
			foreach my $string_id (split (',', $h{string_id})) {	    					
				if (is_int($string_id)) {
	                # It's error code - we will get error message 
	                push @statuses,string(
	                	HSPC::ErrorCodes::get_error_message( $string_id )
	                );
	            } else {
	                # It's localized string code
	                push @statuses,string($string_id);
	            }				
			};
		}
	}	
	return sw_web_parse_template($class,'status_message.tmpl',{
		statuses=>\@statuses,
		is_warning=>$h{is_warning},
	});	
}

### >  error_id
### <  string
sub _error_id_2_str{
	my $error_id = shift;
#	commented (not deleted) for educationaly  purpose
#	my $str = sprintf("%s %i%03i: %s",
#		string('error'),
#		get_module_code($error_id),
#		get_low_error_code($error_id),
#		$ENV{error_obj}->get_error_message($error_id),
#	);	
	my $str = $ENV{error_obj}->get_error_message($error_id);
	return $str;
}

#  >> class
#	=>  title_id
#	=>  title
#   =>  refresh_url
# comment_obj_id and comment_obj_type should both undef or both required
# => comment_obj_id - Object ID
# => comment_obj_type - Object TypeID
#   =>> devscreen_id : DevScreenID
#				for layout apps : app_id-page_id#field_id, look HelpManager doc 
#				for cgi-scipts : unit_id-app_id-page_id#field_id
#   =>> icon : 
#			file name for icon, example : '/pcc/images/i_commercemanager.gif',ask nomad@ about your icon
#          or 
#   		icon_id from customization repository, ask nomad@ about your ID
#   =>  post_root : 1(default)=add "Top" link to path, 0=do not add "Top"
#	=>  path : [{title=>,url=>},{title_id=>,url=>}...]
#   =>  top_target : "target" attribute for Top link in path, i.e. 'fr_right'
#	<=  result
sub title{
	my $class = shift;		
	my %h=(
		post_root=>1,
		@_
	);

	sw_die("comment_obj_id and comment_obj_type should both undef or both required") unless ( ($h{comment_obj_id} && $h{comment_obj_type}) || (!$h{comment_obj_id} && !$h{comment_obj_type}) );
	
	# copy path #
	if( defined $h{path} ){
		my @path = @{$h{path}};
		$h{path} = \@path;
	}
	
	$h{help_debug} = $CONFIG{HELP_DEBUG};	
	$h{tool_id} = $ENV{HSPC_SYSTEM}->tool()->id();

	if( defined $h{devscreen_id} ){
		sw_die('ERROR : title() : devscreen=> can`t be empty') if $h{devscreen_id} eq '';		
		$h{helpscreen_id} = $ENV{HSPC_SYSTEM}->generate_helpscreen_id(devscreen_id=>$h{devscreen_id});
		if ( $ENV{layout} && $h{devscreen_id}) {
			$ENV{layout}->devscreen_id($h{helpscreen_id});
		}

		if (my $plugin_help_url = HSPC::PluginHelper::Base->get_help_url(
				devscreen_id => $ENV{layout} ? $ENV{layout}->devscreen_id : $h{devscreen_id}
			)
		) {
			$h{help_url} = $plugin_help_url;
		} elsif ( $h{tool_id} =~ /^(pcc|rcc)$/ ) {
			## Control center: pcc,rcc
			$h{help_url} = $ENV{COMPREP}->link(alias => 'cc_core_helper', {
				sc_id => $h{helpscreen_id}
			});
		} else {
			## try to find user customized help file
			my $file_path = &sw_find_custom_file(subdir=>'help/',
				file=>"$h{helpscreen_id}.shtml",is_relative_path=>1
			);
			if($file_path) {
				$h{help_url} = '/custom/help/'.$file_path;
			} else {
				$file_path = &sw_find_custom_file(
					base_path=>$CONFIG{HSPCROOT_ROOT}.'/help/',langdir_uc=>1,
					file=>"$h{helpscreen_id}.shtml",is_relative_path=>1);
				$h{help_url} = '/help/'.$file_path if $file_path;
			}
		}
	}

	if ( 
		defined $ENV{HSPC_SYSTEM} 
		&& $ENV{HSPC_SYSTEM}->tool()->id() =~ /^cp$|^rcc$/
		&& $h{devscreen_id}
	) {
		## Show link Add trouble ticket for CP
		my $account_no = $ENV{security_obj}->account_no() || 
			$ENV{cp_obj}->account_obj->id;

		my $provider_id = HSPC::MT::Core->find_account_vendor_id(
			$account_no
		);
		my $ts = HSPC::MT::UM::TS->find_ts_setup(
			provider_id=>$provider_id
		);
		$h{show_trouble_link} = $ts->is_external && 'ext' || $ts->is_enabled && 'int'
			if $ts;
	}
	
	if(	$h{post_root}==1 
		&& defined $ENV{HSPC_SYSTEM} 
		&& defined $ENV{HSPC_SYSTEM}->tool 
		&& $ENV{HSPC_SYSTEM}->tool->path_root_url ne ''
	){	
		my $root = {
			title_id=>$ENV{HSPC_SYSTEM}->tool->path_root_title_id,
			url=>$ENV{HSPC_SYSTEM}->tool->path_root_url,
			target=>defined $h{top_target} ? $h{top_target} : '_self'
		};
		if( defined $h{path} ){
			splice( @{$h{path}},0,0,$root );			
		}else{
			$h{path} = [$root];
		}
	}
		            	            	            
    $h{title} .= string( $h{title_id})  if $h{title_id}; 

    if(defined $h{icon} && $h{icon}!~m|\.|){
	my $icon_med = $h{icon} . '_med';
		if( $ENV{HSPC_SYSTEM}->ui_image_path(id=>$icon_med, skip_error=>1) ){
    		$h{icon} = $ENV{HSPC_SYSTEM}->ui_image_path(id=>$icon_med);
		} else {
	    	$h{icon} = $ENV{HSPC_SYSTEM}->ui_image_path(id=>$h{icon});		
		}		
	}
    
	return sw_web_parse_template(__PACKAGE__,'title.tmpl',\%h);		    
}


# > class
# => title1  	# REQ_Y #
# => title2  	# REQ_N #
# => change_url # REQ_N #  url for Link "( change )"
# => menu       # REQ_Y # items array
#  
#  menu item : hash with keys 
#      title =>  # REQ_Y #  Item Title
#      url =>    # REQ_Y #  Item URL
#      icon =>   # REQ_Y #  Item Icon : HTML code "src='...'"

sub menu_block{
	my ($self,%h)=@_;
	
	return sw_web_parse_template(__PACKAGE__,'menu_block.tmpl',\%h);		    	
}

##  > class
##	>= title_id
##	>= titles
##  => helpbar_devscreen_id 
sub page_description {
	my ($self,%h)=@_;
	$h{title} = string($h{title_id}) if $h{title_id};
	
	if( $h{helpbar_devscreen_id}){
		## prepare full help id
		$h{helpscreen_id} = $ENV{system_obj}->generate_helpscreen_id(
			devscreen_id=>$h{helpbar_devscreen_id}
		);

		if ( my $tooltip = HSPC::PluginHelper::Base->get_tooltip(
				devscreen_id => $ENV{layout} ?
					$ENV{layout}->devscreen_id : $h{helpbar_devscreen_id}
			)
		) {
			$h{title} = $tooltip;
		} else {
			### load help
			$h{title} = HSPC::MT::Core::Web->find_help(
				provider_id=>$ENV{system_obj}->provider_id, 
				helpscreen_id=>$h{helpscreen_id},
				type=>'on_screen',
			);			
			### check editable
			$h{_is_changeable} = $self->is_helpbar_editable;
			if( $h{_is_changeable} ){
				$h{_change_url} = "/cp/index.cgi/edit_hint/screenid,$h{helpscreen_id}";
			}
		}
	}
	
	return '' unless length( $h{title} ) || $h{_is_changeable} ;
	return sw_web_parse_template(__PACKAGE__,'page_description.tmpl',\%h);		    	
}	

# checks whether current user is allowed to edit helpbar content
# > self | class (doesn't matter)
sub is_helpbar_editable {
	my $is_helpbar_editable = 0;
	if( $ENV{system_obj}->tool_id eq 'cp'){
		my $sec_accounts = $ENV{security_obj}->get_accounts(
			person_id=>$ENV{security_obj}->auth_person_id()
		);
		my $prov_id = $ENV{system_obj}->provider_id;

		return
			exists $sec_accounts->{&SW_HSP} &&
				scalar(@{$sec_accounts->{&SW_HSP}}) ? 1 :
			exists $sec_accounts->{&SW_RESELLER} &&
				scalar(grep {$_ == $prov_id} @{$sec_accounts->{&SW_RESELLER}}) ? 1 : 0;
	}
}

1;
